/*  ML_genedata, a program for maximum likelihood modelling of gene family presence/absence data
 *  Copyright (C) 2008 by Matthew Spencer
 *
 *  This program is free software; you may redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  For any other enquiries write to Matthew Spencer, School of Biological Sciences,
 *  University of Liverpool, Crown Street, Liverpool, L69 7ZB, UK.
 *
 *  Or send E-mail to m.spencer@liverpool.ac.uk
 *
 *
 *
 *---------------------------------------------------------------------------
 * Requires the Gnu Scientific Library (http://www.gnu.org/software/gsl/)
 *
 *---------------------------------------------------------------------------
 * Parts of this program (functions to read phylip file) are based on fastDNAml by Gary J. Olsen, which is in turn based
 * partly on dnaml by Joseph Felsenstein
 *
 * Copyright notice from dnaml:
 *
 *     version 3.3. (c) Copyright 1986, 1990 by the University of Washington
 *     and Joseph Felsenstein.  Written by Joseph Felsenstein.  Permission is
 *     granted to copy and use this program provided no fee is charged for it
 *     and provided that this copyright notice is not removed.
 *
 *----------------------------------------------------------------------------
 *
 * Parts of this program (functions to read tree file and data structures for trees) are based on Seq-Gen by Andrew Rambaut and Nick Grassly * Copyright notice for Seq-Gen:
 *  
   Sequence Generator - seq-gen, version 1.3.2
   Copyright (c)1996-2004, Andrew Rambaut & Nick Grassly
   Department of Zoology, University of Oxford			
   All rights reserved.                          

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote 
        products derived from this software without specific prior written 
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


   Any feedback is very welcome.
   http://evolve.zoo.ox.ac.uk/software/Seq-Gen/
   email: andrew.rambaut@zoo.ox.ac.uk
*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_rng.h>
#include "global.h"
#include "treefile.h"
#include "opt_funcs.h"

void ReadParams(int argc, char **argv, parameters *params, char *infilename, char *outfilename, char *treefilename, char *pfilename);
void printHelp(void);
genomedata *make_genomedata(int ntaxa, int ngenes, int nhidden);
void free_genomedata(genomedata *gd);
int set_nhidden(int ntaxa,parameters *params);
void generate_hidden(genomedata *gd,parameters *params);
genomedata *read_genomedata(char *filename);
boolean whitechar (int ch);
boolean bindigitchar (int ch);
boolean getdata (genomedata *gd,FILE *infile);
void printgd(genomedata *gd);
int match_patterns(genomedata *gdn, genomedata *gd,parameters *params);
TTree **read_treefile(FILE *tree_fv, int numTrees, int ngenes, int nstates, int gammacat);
void free_treeSet(TTree **treeSet, int numTrees, int ngenes, int ncat, int gammacat);
void fill_treedata(genomedata *gd, TTree **treeSet, int ncat);
int setuprates(TTree **treeSet, int numTrees,int ngammacat);
double ***make_Qset(int nQ, int nstate);
void free_Qset(double ***Qset, int nQ, int nstate);
void test_Qset(double ***Qset);
void test_rootset(double **rootset);
void initparms(parameters *params,char *pfilename,double ***Qset,double *rhop,double **rootp, int ncat, int maxQ,double *alpha);
void PfromQ22(double **P, double ***Qset, int i, double t, double *rmult, int c, double *edgescale);
double **make_P(int nstate);
void free_P(double **P, int nstate);
double ***make_Panc(int nstate, int ncat);
void free_Panc(double ***Panc, int nstate, int ncat);
void print_P(double **P, int nstate, double t);
double **make_piset(int nQ, int nstate);
double *make_rho(int ncat);
void free_piset(double **piset, int nQ);
void print_pi(double **piset, int nstate, int i);
void treelike(TTree *tree, TNode *node, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, int i, double *rmult, double *edgescale);
void get_treelike_cat(TTree *tree, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double **rootset, int whichcat, double *rmult, double *edgescale);
double vertexlike(TTree *tree, TNode *node, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double *rho, genomedata *gd, double **rootset, double *rmult, double *edgescale);
double vertexlike_subtrees_root(TTree *tree, TNode *node, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double *rho, genomedata *gd, double **rootset, int LRind, double *rmult, double *edgescale);
double cond_like(TTree *tree,genomedata *gd,int ncat);
void update_branchULike(TNode *node,int ngenes,int nstates,int ncat,double ***Qset,double **Pleft,double **Pright,int LRind, double *rmult, double *edgescale);
void update_ULike(TNode *node, int ngenes, int nstates, int ncat);
void print_UCLike(TNode *node, int ngenes, int nstates, int ncat);
void write_results(FILE *out_fv,TTree *tree, double treeloglike,parameters *params,int ncat,genomedata *gd,double *rho,double etime,double ewall,int conv,int optit,int notobs,int ngenes, int nmajor, int gammacat);
void write_settings(parameters *params,char*outfilename,char *pfilename);
void write_pprobs(FILE *out_fv,genomedata *gd, double **likes, double *rho, int ncat, int nmajor, int gammacat);
double gamma_rate(double alpha, int k, int i);
void testPfromQ22(double **P, double ***Qset, double *rmult, double *edgescale);
void simulate_data(parameters *params, char *treefilename, char *outfilename, char *pfilename, int argc, char **argv);
void generate_categories(int ncat,double *rho,int sgf,double *catvec,gsl_rng *r, int i);
void write_simdata(FILE *out_fv,int numTaxa,int sgf,TTree *tree);
int discrete_rand(double *rho, int ncat,gsl_rng *r);
void generate_rootstate(double **rootset, int sgf,double *statevec,double *catvec,gsl_rng *r,int i);
void generate_vertexstates(int sgf, double *statevec,double *catvec,TNode *node,gsl_rng *r,double **Pleft,double **Pright,double ***Qset,double *rmult,double *edgescale,int i);
int check_observability(int numTaxa, TTree *tree, int i, parameters *params);

/*read parameters*/
void ReadParams(int argc, char **argv, parameters *params, char *infilename, char *outfilename, char *treefilename, char *pfilename)
{
  int c,offset=0;
  extern char *optarg;
  params->printHelp=FALSE;
  params->quiet=TRUE;
  params->printlikes=FALSE;
  params->edgeopt=TRUE;
  params->startparms=FALSE;
  params->ctype=0;
  params->printdata=FALSE;
  params->optchoice=QNEWT;
  params->parmopt=TRUE;
  params->gammacat=1;
  params->edgescale=1.;
  params->simulate=FALSE;
  params->sgf=0;
  opterr = 0;
  while ((c = getopt (argc, argv, "dqnlhfs:p:c:g:o:m:")) != -1) switch (c){
  case 'd':/*print genome data*/
    params->printdata=TRUE;
  case 'q':/*quiet operation*/
      params->quiet=TRUE;
    break;
  case 'm':/*generate simulated data*/
    params->simulate=TRUE;
    params->sgf=atoi(optarg);/*number of simulated gene families*/
    offset=-1;/*we won't need an input data file*/ 
   break;
  case 'n':/*noisy operation*/
    params->quiet=FALSE;
    break;
  case 'l':/*print gene family likelihoods*/
    params->printlikes=TRUE;
    break;
  case 'h':/*help message*/
    params->printHelp=TRUE;
    return;
    break;
  case 'f':/*don't optimize edge lengths*/
    params->edgeopt=FALSE;
    break;
  case 's':/*scale edge lengths by a constant*/
    params->edgescale=atof(optarg);
    if(params->edgescale<0.){
	fprintf(stderr,"Invalid edge scale factor: %s\n",optarg);
	exit(EXIT_FAILURE);
    }
    break;
  case 'p':/*read starting parameters from file*/
    params->startparms=TRUE;
    sprintf(pfilename,"%s",optarg);
    break;
  case 'c':/*conditioning on observability*/
    if((*optarg=='0')||(*optarg=='1')||(*optarg=='2')||(*optarg=='3') ) params->ctype=atoi(optarg);/*implemented conditioning cases*/
    else{
      fprintf (stderr,"Invalid conditioning setting: %s\n",(char*)optarg);
      exit(EXIT_FAILURE);
    }
    break;
  case 'o':/*optimization method*/
    if(!strcmp(optarg,"S")) params->optchoice=SIMPLEX;
    else if(!strcmp(optarg,"Q")) params->optchoice=QNEWT;
    else if(!strcmp(optarg,"N")) params->parmopt=FALSE;/*no parameter optimization*/
    else{
      fprintf(stderr,"Invalid estimation method: %s\n",optarg);
      exit(EXIT_FAILURE);
    }
    break;
    case 'g':
      params->gammacat=atoi(optarg);
      if(params->gammacat<=0){
	fprintf(stderr,"Invalid number of gamma categories: %s\n",optarg);
	exit(EXIT_FAILURE);
      }
      break;
  case '?':
    if (isprint (optopt))
      fprintf (stderr, "Unknown option `-%c'.\n", optopt);
    else
      fprintf (stderr,"Unknown option character `\\x%x'.\n",optopt);
    exit(EXIT_FAILURE);
  default:
    abort ();
}
  if(offset==0){
    if(argc > optind){
      sprintf(infilename,"%s",argv[optind]);
    }
    else{/* ask for the input-file */
      printf("\nEnter input file : ");
      scanf("%s",infilename);
    }
  }
  if(argc>(optind+1+offset)){
    sprintf(outfilename,"%s",argv[optind+1+offset]);
  }
  else{/* ask for output-file.        */
    printf("\nEnter output file : ");
    scanf("%s",outfilename);
  }
  if(argc>(optind+2+offset)){
    sprintf(treefilename,"%s",argv[optind+2+offset]);
  }
  else{/* ask for tree file.        */
    printf("\nEnter tree file : ");
    scanf("%s",treefilename);
  }
}

/*display help message*/
void printHelp(){
  printf("\n"); 
  printf(PROGRAM_NAME);
  printf(" ");
  printf(PROGRAM_VERSION);
  printf(": maximum likelihood methods for gene content data\n"); 
  printf("Usage: ");
  printf(PROGRAM_NAME);
  printf(" [-h] [-d] [-q] [-n] [-l] [-f] [-p pfile] [-c#] [-g#] [-s#] [-o:] [-m#] [infile] [outfile] [treefile] \n");
  printf("Options: \n");
  printf("-h: this help message\n");
  printf("-d: print out genome data and unobservable patterns. Note that patterns that occur more than once will only be printed as a single column, but with a weight greater than 1. The row labelled 'Matches' reports the zero-indexed column of the printed data to which each gene family corresponds.\n");
  printf("-q: quiet operation (default)\n");
  printf("-n: noisy operation: settings, progress, etc.\n");
  printf("-l: include gene family likelihoods and empirical Bayes posterior probabilities for each category and gene family in output file\n");
  printf("-f: fix edge lengths at input values\n");
  printf("-p file: read starting parameters (other than edge lengths) from file pfile with the following entries:\n");
  printf("\tRelative rates of gene gain and loss for each rate matrix\n\t(these will be rescaled to one expected event per unit time at equilibrium).\n");
  printf("\tMixing probability for each category.\n");
  printf("\tProbability of gene family absence at root for each category.\n");
  printf("\tShape parameter alpha for gamma distribution (has no effect unless -g used with more than one rate class).\n");
  printf("\tfor example,\n\t\t1.2 0.1\n\t\t0.1 0.7\n\t\t0.25 0.75\n\t\t0.3 0.4\n\t\t0.5\n");
  printf("\tspecifies relative gene gain and loss rates of 1.2 and 0.1 for the first rate matrix,\n\trelative gain and loss rates of 0.1 and 0.7 for the second rate matrix,\n\tmixing probabilities of 0.25 and 0.75 for two categories,\n\tprobabilities of gene absence at the root of 0.3 and 0.4 for these two categories, and shape parameter 0.5.\n");
  printf("-c#: setting for conditioning on observable gene families. Options are 0 (all gene families are observable, default), 1 (condition on presence in at least one genome), 2 (condition on presence in at least two genomes), 3 (condition on presence in at least three genomes). Any gene family that appears in the data set but does not meet the observability criterion will have a weight of zero when calculating the likelihood and estimating parameters.\n");
  printf("-g#: number of gamma rate categories (default 1).\n");
  printf("-s#: constant by which to multiply all edge lengths when calculating likelihoods (default 1.0). Must be non-negative.\n");
  printf("-o: setting for optimization method (parameters other than edge lengths). Options are S (simplex), Q (quasi-Newton, default) and N (no parameter optimization).\n");
  printf("-m# : generate the specified number of simulated gene families, instead of using data in input file.\n");
  printf("The output from -m is a phylip-format file (readable as an input file by this program), with rows at the end containing the root state, the true category for each gene family, and the command used to generate the data.\n");
  printf("Can be used with -p, -c, -g and -s.\n");
  printf("Seeds the pseudorandom number generator with system time, so use a sleep command between reps if running multiple times from a script.\n");
  printf("Input file format: a phylip format data matrix, arranged like an alignment (no input file with -m option).\n");
  printf("Taxon names must be no more than 10 characters,\n");
  printf("and should not contain white space.  Number of taxa and gene families on the first row.  0 (absence) and 1 (presence) are the only symbols allowed.\n");
  printf("The output is a text file containing:\n\t-The tree topology with initial edge lengths, the rate matrix index for each category on each edge, and the initial parameter estimates\n\t-The log likelihood, gene family log likelihoods (if -l specified), and empirical Bayes posterior probabilities and category with maximum posterior probability (if -l specified and more than one category). ");
  printf("These are conditional on observability, and so are not meaningful for any gene families that appear in the data file but do not meet the specified observabilty criterion (-c). Such gene families are ignored when calculating the likelihood. The output includes a 0/1 vector where 0 indicates an unobservable and 1 an observable gene family.");
  printf("\n\t-The estimated edge lengths and parameters (if these were not fixed).\n");
  printf("Tree file format: a Newick tree with initial edge lengths. Subsequent lines (one per category of gene families) must contain the same tree topology, with the indices of the rate matrix used on that edge in that category replacing the edge lengths.\n"); 
  printf("The indices should be non-negative integers, and you should use all the integers between zero and the largest index. For example, use indices {0,1,2} rather than {1,2,3} or {0,1,3}.\n"); 
  printf("There is no ordering associated with the indices: rate matrix 1 does not imply higher rates than rate matrix 0. All rate matrices are scaled so that there is one expected event per unit time at equilibrium, but they will differ in the relative rates of gene gains and losses. The number of rate matrices does not have to be the same as the number of categories, and a rate matrix may appear in multiple categories.\n");
  printf("Copyright (C) 2008 Matthew Spencer, School of Biological Sciences, University of Liverpool (m.spencer@liverpool.ac.uk).\n");

}

/*read a phylip-format file of gene presence-absence*/
/*return a pointer to a structure containing the data*/
genomedata *read_genomedata(char *filename)
{
	FILE *infile;
	int ntaxa,ngenes;
	genomedata *gd;
	
	/*open the file*/
	if( (infile=fopen(filename,"r")) == NULL)
	{
    		fprintf(stderr,"File not found:%s\n",filename);
    		exit(EXIT_FAILURE);
  	}
	fscanf(infile,"%d%d",&ntaxa,&ngenes);
	gd=make_genomedata(ntaxa,ngenes,0);/*no hidden patterns in what we read in*/

	/*read in the presence/absence data*/
	if (!getdata (gd,infile)){
		fclose(infile);
		exit(EXIT_FAILURE);
	}

	fclose(infile);
	return gd;
}

/*allocate memory for a structure containing genome data. ntaxa and ngenes are number of taxa and number of observable genes, and nhidden is number of unobservable patterns that we might want space for*/
genomedata *make_genomedata(int ntaxa, int ngenes, int nhidden)
{
	genomedata *gd;
	int i,ncol;

	ncol=ngenes+nhidden;
	gd=(genomedata *)calloc(1,sizeof(genomedata));

	gd->gdata=(int **)calloc(ntaxa,sizeof(int*));
	if(gd->gdata==NULL){
	  (void)fprintf(stderr,"ERROR: calloc failed");
	  (void)exit(EXIT_FAILURE);
	}
	gd->names=(char**)calloc(ntaxa,sizeof(char*));
	if(gd->names==NULL){
	  (void)fprintf(stderr,"ERROR: calloc failed");
	  (void)exit(EXIT_FAILURE);
	}

	for(i=0;i<ntaxa;i++){
	  gd->gdata[i]=(int*)calloc(ncol,sizeof(int));
	  if(gd->gdata[i]==NULL){
	    (void)fprintf(stderr,"ERROR: calloc failed");
	    (void)exit(EXIT_FAILURE);
	  }
	  gd->names[i]=(char*)calloc(nmlngth+1,sizeof(char));
	  if (gd->names[i]==NULL){
	    (void)fprintf(stderr,"ERROR: calloc failed");
	    (void)exit(EXIT_FAILURE);
	  }
	}
	if((gd->weights=(int*)calloc(ncol,sizeof(int)))==NULL){
	  (void)fprintf(stderr,"ERROR: calloc failed");
	  (void)exit(EXIT_FAILURE);
	}
	if((gd->match=(int*)calloc(ncol,sizeof(int)))==NULL){
	  (void)fprintf(stderr,"ERROR: calloc failed");
	  (void)exit(EXIT_FAILURE);
	}
	gd->ntaxa=ntaxa;
	gd->ngenes=ngenes;
	gd->nunique=ngenes;/*will change this later if we match patterns*/
	gd->nhidden=nhidden;
	for(i=0;i<ncol;i++) gd->weights[i]=1;
	return gd;
}

/*free memory for structure containing genome data*/
void free_genomedata(genomedata *gd)
{
	int i;
	for(i=0;i<gd->ntaxa;i++){
	  free(gd->gdata[i]);
	  free(gd->names[i]);
	}
	free(gd->gdata);
	free(gd->names);
	free(gd->weights);
	free(gd->match);
	free(gd);
}

/*how many unobservable patterns?*/
int set_nhidden(int ntaxa,parameters *params)
{
  int nh;
  switch(params->ctype){
  case 0:/*no unobservable patterns*/
    nh=0;
    break;
  case 1:/*zeros everywhere unobservable*/
    nh=1;
    break;
  case 2:/*zeros everywhere, and zeros everywhere except one genome, unobservable*/
    nh=1+ntaxa;
    break;
  case 3:/*zeros everywhere, everywhere except one, and everywhere except two genomes, all unobserbable*/
    nh=1+ntaxa+ntaxa*(ntaxa-1)/2;
    break;
  default:
    nh=0;
  }
  return(nh);

}/*set_nhidden*/

/*generate unobservable patterns*/
void generate_hidden(genomedata *gd,parameters*params)
{
  int i,j,k,c;

  switch(params->ctype){
  case 0:/*no unobservable patterns*/
    break;
  case 1:/*zeros everywhere unobservable*/
    for(i=0;i<gd->ntaxa;i++) gd->gdata[i][gd->nunique]=0;/*put a column of zeros at the end of the data*/
    break;
  case 2:/*zeros everywhere, and everywhere except one genome, unobservable*/
    for(i=0;i<gd->ntaxa;i++){
      gd->gdata[i][gd->nunique]=0;/*put a column of zeros at the end of the data*/
      for(j=0;j<gd->ntaxa;j++){
	gd->gdata[i][gd->nunique+1+j]=(i==j);/*put a one on the diagonal*/
      }
    }
    break;
  case 3:/*zeros everywhere, everywhere except one genome, and everywhere except two genomes, unobservable*/
    for(i=0;i<gd->ntaxa;i++){
      c=0;
      gd->gdata[i][gd->nunique]=0;/*put a column of zeros at the end of the data*/
      for(j=0;j<gd->ntaxa;j++){
	gd->gdata[i][gd->nunique+1+j]=(i==j);/*put a one on the diagonal*/
	for(k=j+1;k<gd->ntaxa;k++){/*fill in all patterns with two ones*/
	  gd->gdata[i][gd->nunique+1+gd->ntaxa+c]=(i==j);
	  gd->gdata[k][gd->nunique+1+gd->ntaxa+c]=1;
	  c++;
	}
      }
    }
    break;
  default:
    break;
  }
}/*generate_hidden*/

/*read a 0/1 data matrix in phylip format: modified from fastDNAml*/
boolean getdata (genomedata *gd,FILE *infile)
  {
    int   i, j, k, basesread, basesnew, ch;
    char *nameptr;
    boolean  allread, firstpass;


    basesread = basesnew = 0;

    allread = FALSE;
    firstpass = TRUE;
    ch = ' ';

    while (! allread) {
      for (i = 0; i < gd->ntaxa; i++) {     /*  Read data line */

        if (firstpass) {                      /*  Read species names */
          j = 0;
          while (whitechar(ch = getc(infile))) {  /*  Skip blank lines */
            if (ch == '\n')  j = 0;  else  j++;
            }

          if (j > nmlngth) {
            printf("ERROR: Blank name for species %d; ", i+1);
            printf("check number of species,\n");
            printf("       number of gene families, and interleave option.\n");
            return  FALSE;
            }

          nameptr = gd->names[i];
          for (k = 0; k < j; k++)  *nameptr++ = ' ';

          while (ch != '\n' && ch != EOF && !whitechar(ch)) {/*MS: added !whitechar(ch) 8/7/08: then don't need to pad names to correct length with white space*/
            if (whitechar(ch))  ch = ' ';
            *nameptr++ = ch;
            if (++j > nmlngth) break;
            ch = getc(infile);
            }

          while (*(--nameptr) == ' ') ;          /*  remove trailing blanks */
          *(++nameptr) = '\0';                   /*  add null termination */
          if (ch == EOF) {
            printf("ERROR: End-of-file in name of species %d\n", i+1);
            return  FALSE;
            }
          }    /* if (firstpass) */

        j = basesread;
        while ((j < gd->ngenes)
               && ((ch = getc(infile)) != EOF)
               && (ch != '\n')) {
          if (bindigitchar(ch)) {
            if(ch=='0')
		gd->gdata[i][j]=0;
            else
		gd->gdata[i][j]=1;
	    gd->weights[j]=1;
            j++;
            }
          else if (whitechar(ch)) ;
          else {
            printf("ERROR: Bad character (%c) at gene family %d of genome %d\n",
                    ch, j+1, i+1);
            return  FALSE;
            }
          }

        if (ch == EOF) {
          printf("ERROR: End-of-file at gene family %d of genome %d\n", j+1, i+1);
          return  FALSE;
          }

        if (! firstpass && (j == basesread)) i--;        /* no data on line */
        else if (i == 0) basesnew = j;
        else if (j != basesnew) {
          printf("ERROR: genomes out of alignment\n");
          printf("%d (instead of %d) gene families read in genome %d\n",
                  j - basesread, basesnew - basesread, i+1);
          return  FALSE;
          }

        while (ch != '\n' && ch != EOF) ch = getc(infile);  /* flush line */
        }                                                  /* next sequence */
      firstpass = FALSE;
      basesread = basesnew;
      allread = (basesread >= gd->ngenes);
      }
  return TRUE;
  } /* getdata */


boolean whitechar (int ch)
{ 
	return (ch == ' ' || ch == '\n' || ch == '\t');
}

boolean bindigitchar (int ch)
{
	return (ch >= '0' && ch <= '1');
}

/*print out the data.*/
void printgd(genomedata *gd)
{
  int i,j;
  
  printf("Data\n %d %d\n",gd->ntaxa,gd->nunique);
  for(i=0;i<gd->ntaxa;i++){
    printf(gd->names[i]);
    printf("\t");
    for(j=0;j<gd->nunique;j++)
      printf("%d ",gd->gdata[i][j]);
    printf("\n");
  }
  printf("weights\t");
  for(j=0;j<gd->nunique;j++)
    printf("%d ",gd->weights[j]);
  printf("\n");
  printf("Matches\t");
  for(j=0;j<gd->ngenes;j++)
    printf("%d ",gd->match[j]);
  printf("\n");

  if(gd->nhidden>0){/*display unobservable patterns*/
    printf("Unobservable patterns\n %d %d\n",gd->ntaxa,gd->nhidden);
    for(i=0;i<gd->ntaxa;i++){
      printf(gd->names[i]);
      printf("\t");      
      for(j=0;j<gd->nhidden;j++)
	printf("%d ",gd->gdata[i][j+gd->nunique]);
      printf("\n");
    }
  }
  printf("\n");
}/*printgd*/

/*identify non-unique patterns in gdn. Copy unique patterns and weights to gd. Identify and give zero weight to patterns we set as unobservable (returns number of patterns that were seen that were set as unobservable).*/
int match_patterns(genomedata *gdn, genomedata *gd, parameters *params)
{
  int c,i,j,k,obs,notobs,r,s;

   c=0;
   notobs=0;
   for(i=0;i<gdn->ngenes;i++){
    j=0;
    s=1;
    r=0;
    for(k=0;k<gdn->ntaxa;k++) r+=gdn->gdata[k][i];/*number of occurrences*/
    notobs+=!(obs=(r>=params->ctype));/*is this gene family observable according to our setting?*/
    while((j<c) & (s!=0)){
      k=0;
      r=0;
      while((r==0) & (k<gdn->ntaxa)){
	if(gdn->gdata[k][i]!=gd->gdata[k][j]) r=1;
	k++;
      }
      s=r;
      j++;
    }
    if(s==0){/*found matching pattern*/
      gd->weights[j-1]+=obs;/*add to number of observations of matching pattern (unless pattern shouldn't be observable)*/
      gd->match[i]=j-1;/*index of matching pattern*/
    }
    else{/*no matching pattern*/
      gd->weights[c]=obs;
      gd->match[i]=c;
      for(k=0;k<gd->ntaxa;k++) gd->gdata[k][c]=gdn->gdata[k][i];
      c++;
    }
  }
   for(k=0;k<gdn->ntaxa;k++) strcpy(gd->names[k],gdn->names[k]);/*copy taxon names*/
  gd->nunique=c;
  return notobs;
}/*match_patterns*/




/*read the set of trees into a structure*/
TTree **read_treefile(FILE *tree_fv, int numTrees, int ngenes, int nstates,int gammacat)
{
  int i,treeNo,numTaxa=0;
  TTree **treeSet;


  /*allocate memory for tree*/
  treeSet = (TTree **)malloc(sizeof(TTree **) * numTrees);
  if (treeSet==NULL) {
    fprintf(stderr, "Out of memory\n");
    exit(0);
  }
	
  for (i = 0; i < numTrees; i++)
    if ((treeSet[i]=NewTree())==NULL) {
      fprintf(stderr, "Out of memory\n");
      exit(0);
    }

  /*read trees from file with list of rate matrices on each edge in each partition*/
  /*numTrees tells us number of partitions*/
  treeNo=0;
  do {
    /*printf("Reading tree 0\n");*/
    ReadTree(tree_fv, treeSet[0], treeNo+1, 0, NULL, ngenes, nstates, gammacat*numTrees,1);/*1 means allocate space for likelihoods*/
    if (treeNo==0)
      numTaxa=treeSet[0]->numTips; 
    else if (treeSet[0]->numTips != numTaxa) {
      fprintf(stderr, "All trees must have the same number of tips.\n");
      exit(0);
    }
    i = 1;
		while (i < numTrees) {
		  /*printf("Reading tree %d\n",i);	*/
			if (!IsTreeAvail(tree_fv)) {
			  fprintf(stderr, "\nNot enough trees in tree file\n");
				exit(0);
			}

			ReadTree(tree_fv, treeSet[i], treeNo+1, treeSet[0]->numTips, treeSet[0]->names, ngenes, nstates, gammacat*numTrees,0);/*0 means don't allocate space for likelihoods*/
			if (treeSet[i]->numTips != numTaxa) {
				fprintf(stderr, "All trees must have the same number of leaves.\n");
				exit(0);
			}
			i++;
		}
    treeNo++;
  } while (IsTreeAvail(tree_fv));
  return treeSet;
}/*read_treefile*/

/*free the memory used to store the trees*/
void free_treeSet(TTree **treeSet, int numTrees, int ngenes, int ncat, int gammacat)
{
  int i;

  for (i = 0; i < numTrees; i++) FreeTree(treeSet[i],ngenes, ncat, gammacat);
  free(treeSet);
}/*free_treeSet*/

/*initialize the conditional likelihoods at the leaves of the tree*/
void fill_treedata(genomedata *gd, TTree **treeSet, int ncat)
{
  int i,j,k,c;

  /*find the taxon in the tree*/
  for(i=0;i<gd->ntaxa;i++){
    j = 0;
    while (j < gd->ntaxa && strcmp(gd->names[i], treeSet[0]->names[j]) != 0) j++;
    if (j == gd->ntaxa) {
      fprintf(stderr, "Taxon names in tree and data array do not match.\n");
      exit(0);
    }
    /*fill in the data*/
    for(k=0;k<(gd->nunique+gd->nhidden);k++){
      for(c=0;c<ncat;c++){
	treeSet[0]->tips[j]->CLike[c][gd->gdata[i][k]][k]=1;/*conditional likelihood of observed data at leaf*/
      }
    }
  }
}/*fill_treedata*/

/*copy indices stored as edge lengths on trees other than treeSet[0] to the rate matrix indices in treeSet[0]. Return largest Q matrix index used.*/
int setuprates(TTree **treeSet, int numTrees, int gammacat)
{
  int i,j,maxQ=0;
  for(i=1;i<numTrees;i++)
    for(j=0;j<gammacat;j++) fill_Qind(treeSet[0]->root,treeSet[i]->root,gammacat*(i-1)+j,&maxQ);
  maxQ=maxQ+1;/*assume Q indices start at zero*/
  return maxQ;
}/*setuprates*/

/*allocate memory for rate matrices*/
double ***make_Qset(int nQ, int nstate)
{
  int i,j;
  double ***Qset;

  Qset=(double ***)calloc(nQ,sizeof(double**));
	if(Qset==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed: make_Qset");
		(void)exit(EXIT_FAILURE);
	}

	for(i=0;i<nQ;i++){
		Qset[i]=(double**)calloc(nstate,sizeof(double*));
		if(Qset[i]==NULL){
			(void)fprintf(stderr,"ERROR: calloc failed: make_Qset");
			(void)exit(EXIT_FAILURE);
		}
		for(j=0;j<nstate;j++){
		  Qset[i][j]=(double*)calloc(nstate,sizeof(double));
		  if(Qset[i][j]==NULL){
			(void)fprintf(stderr,"ERROR: calloc failed: make_Qset");
			(void)exit(EXIT_FAILURE);
		  }
		}
	}
	return(Qset);
}/*make_Qset*/

/*free memory for rate matrices*/
void free_Qset(double ***Qset, int nQ, int nstate)
{
  int i,j;
  for(i=0;i<nQ;i++){
    for(j=0;j<nstate;j++){
      free(Qset[i][j]);
    }
    free(Qset[i]);
  }
  free(Qset);
}/*free_Qset*/

/*allocate memory for stationary probabilities*/
/*rows are rate matrices, columns are states*/
double **make_piset(int nQ, int nstate)
{
  int i;
  double **piset;

  piset=(double**)calloc(nQ,sizeof(double*));
  if(piset==NULL){
    (void)fprintf(stderr,"ERROR: calloc failed: make_piset");
    (void)exit(EXIT_FAILURE);
  }
  for(i=0;i<nQ;i++){
    piset[i]=(double*)calloc(nstate,sizeof(double));
    if(piset[i]==NULL){
      (void)fprintf(stderr,"ERROR: calloc failed: make_piset");
      (void)exit(EXIT_FAILURE);
    }
  }
  return(piset);
}/*make_piset*/

/*free memory for stationary probabilities*/
void free_piset(double **piset,int nQ)
{
  int i;
  for(i=0;i<nQ;i++) free(piset[i]);
  free(piset);
}/*free_piset*/

/*allocate memory for mixing probabilities*/
double *make_rho(int ncat)
{
  double *rho;
  if((rho=(double*)calloc(ncat,sizeof(double)))==NULL){
      (void)fprintf(stderr,"ERROR: calloc failed: make_piset");
      (void)exit(EXIT_FAILURE);
  }
  return(rho);
}/*make_rho*/

/*print mixing parameters for major categories and mixing probs for major categories * rate classes*/
void print_rho(double *rho, double *rhop, int gammacat, int nmajor)
{
  int i,j;

  printf("Major category mixing probs:\n");
  for(i=0;i<nmajor;i++) printf("%f ",rhop[i]);
  printf("\nCategory mixing probs:\n");
  for(i=0;i<nmajor;i++){
    printf("Major category %d\n",i);
    for(j=0;j<gammacat;j++){
      printf("Gamma class %d, mixing prob %f\n",j,rho[i*gammacat+j]);
    }
  }
  printf("Overall:\n");
  for(i=0;i<nmajor*gammacat;i++) printf("%f ",rho[i]);
  printf("\n");
}/*print_rho*/

/*print root probability of state zero in each major category * rate class*/
void print_root(double **rootset, double **rootp, int gammacat, int nmajor,int nstate)
{
  int i,j,k;

  printf("Major category root probs:\n");
  for(i=0;i<nmajor;i++)
    for(k=0;k<nstate;k++) printf("%f ",rootp[i][k]);
  printf("\n");
  printf("Category root probs:\n");
  for(i=0;i<nmajor;i++){
    printf("Major category %d:\n",i);
    for(j=0;j<gammacat;j++){
      printf("Gamma class %d: ",j);
      for(k=0;k<nstate;k++) printf("%f ",rootset[i*gammacat+j][k]);
    }
    printf("\n");
  }
  printf("Overall:\n");
  for(i=0;i<nmajor*gammacat;i++){
    for(k=0;k<nstate;k++) printf("%f ",rootset[i][k]);
    printf("\n");
    }
}/*print_root*/

/*allocate memory for P matrix*/
double **make_P(int nstate)
{
  int i;
  double **P;

  P=(double**)calloc(nstate,sizeof(double*));
  if(P==NULL){
    (void)fprintf(stderr,"ERROR: calloc failed: make_P");
    (void)exit(EXIT_FAILURE);
  }
  for(i=0;i<nstate;i++){
    P[i]=(double*)calloc(nstate,sizeof(double));
    if(P[i]==NULL){
      (void)fprintf(stderr,"ERROR: calloc failed: make_P");
      (void)exit(EXIT_FAILURE);
    }
  }
  return(P);
}/*make_P*/

/*free memory for P matrix*/
void free_P(double **P, int nstate)
{
  int i;
  for(i=0;i<nstate;i++) free(P[i]);
  free(P);
}/*free_P*/

/*allocate memory for set of P matrices*/
double ***make_Panc(int nstate, int ncat)
{
  int i;
  double ***Panc;

  Panc=(double***)calloc(ncat,sizeof(double**));
  if(Panc==NULL){
    (void)fprintf(stderr,"ERROR: calloc failed: make_Panc");
    (void)exit(EXIT_FAILURE);
  }
  for(i=0;i<ncat;i++) Panc[i]=make_P(nstate);
  return(Panc);
}/*make_Panc*/

/*free memory for set of P matrices*/
void free_Panc(double ***Panc, int nstate, int ncat)
{
  int i;
  for(i=0;i<ncat;i++) free_P(Panc[i],nstate);
  free(Panc);
}/*free_Panc*/

/*write some test data to the rate matrices*/
/*will only work for 2x2 case*/
void test_Qset(double ***Qset)
{
#ifdef DEBUG/*for testinput.txt, test.tree*/
    Qset[0][0][0]=-1;
    Qset[0][0][1]=1;
    Qset[0][1][0]=2;
    Qset[0][1][1]=-2;

    Qset[1][0][0]=-3;
    Qset[1][0][1]=3;
    Qset[1][1][0]=4;
    Qset[1][1][1]=-4;
#elif defined(DEBUG2)/*for testinput2.txt, test2.tree*/
    Qset[0][0][0]=-0.35;
    Qset[0][0][1]=0.35;
    Qset[0][1][0]=0.35;
    Qset[0][1][1]=-0.35;    
#elif defined(DEBUG3)/*for test_ML_fourtaxa_binary.phy, test_fourtaxa_c.tree*/
    Qset[0][0][0]=-0.35;
    Qset[0][0][1]=0.35;
    Qset[0][1][0]=0.15;
    Qset[0][1][1]=-0.15;    
#elif defined(BRIAN_DEBUG)/*for Bacillus.phy, Bacillus.tree, Brian's case 1*/
    Qset[0][0][0]=-0.51;
    Qset[0][0][1]=0.51;
    Qset[0][1][0]=0.51;
    Qset[0][1][1]=-0.51;
#elif defined(BRIAN_DEBUG2)/*for Bacillus.phy, Bacillus_case2.tree, Brian's case 2*/
    Qset[0][0][0]=-0.35;
    Qset[0][0][1]=0.35;
    Qset[0][1][0]=0.35;
    Qset[0][1][1]=-0.35;

    Qset[1][0][0]=-4.42;
    Qset[1][0][1]=4.42;
    Qset[1][1][0]=4.42;
    Qset[1][1][1]=-4.42;
#elif defined(BRIAN_DEBUG3)/*for Bacillus.phy, Bacillus_case2.tree, Brian's case 2 but scaled Q matrices*/
    Qset[0][0][0]=-1.;
    Qset[0][0][1]=1.;
    Qset[0][1][0]=1.;
    Qset[0][1][1]=-1.;

    Qset[1][0][0]=-1.;
    Qset[1][0][1]=1.;
    Qset[1][1][0]=1.;
    Qset[1][1][1]=-1.;
#endif 
}/*test_Qset*/

/*write some test values to root probabilities*/
void test_rootset(double **rootset)
{
#ifdef DEBUG/*for testinput.txt, test.tree*/
  rootset[0][0]=2./3.;
  rootset[0][1]=1./3.;
  rootset[1][0]=2./3.;
  rootset[1][1]=1./3.;
#elif defined(DEBUG2)/*for testinput2.txt, test2.tree*/
  rootset[0][0]=0.5;
  rootset[0][1]=0.5;
#elif defined(DEBUG3)/*for test_ML_fourtaxa_binary.phy, test_fourtaxa_c.tree*/
  rootset[0][0]=0.15/0.5;
  rootset[0][1]=0.35/0.5;
#elif defined(BRIAN_DEBUG)/*for Bacillus.phy, Bacillus.tree, Brian's case 1*/
  rootset[0][0]=0.5;
  rootset[0][1]=0.5;
#elif defined(BRIAN_DEBUG2)/*for Bacillus.phy, Bacillus_case2.tree, Brian's case 2*/
  rootset[0][0]=0.5;
  rootset[0][1]=0.5;
#elif defined(BRIAN_DEBUG3)/*for Bacillus.phy, Bacillus_case2.tree, Brian's case 2 but scaled Q matrices*/
  rootset[0][0]=0.5;
  rootset[0][1]=0.5;
#endif 
}/*test_rootset*/

/*initialize parameters other than edge lengths*/
void initparms(parameters *params,char *pfilename,double ***Qset,double *rhop,double **rootp, int ncat, int maxQ, double *alpha)
{
  FILE *pfile;
  int i;
  double rhostart;

  if(params->startparms){/*read parameters from file*/
    if( (pfile=fopen(pfilename,"r")) == NULL)/*open the file*/
    {
      fprintf(stderr,"File not found:%s\n",pfilename);
      exit(EXIT_FAILURE);
    }
    for(i=0;i<maxQ;i++){/*read gene loss rates from file*/
      fscanf(pfile,"%lf%lf",&Qset[i][0][1],&Qset[i][1][0]);/*relative rates of gene gain and loss*/
      Qset[i][0][0]=-Qset[i][0][1];
      Qset[i][1][1]=-Qset[i][1][0];
    }
    for(i=0;i<ncat;i++) fscanf(pfile,"%lf",&rhop[i]);/*read mixing probs from file*/
    for(i=0;i<ncat;i++){
      fscanf(pfile,"%lf",&rootp[i][0]);/*read root probs of state zero from file*/
      rootp[i][1]=1.-rootp[i][0];
    }
    fscanf(pfile,"%lf",alpha);
    fclose(pfile);
  }
  else{/*use default values*/
    rhostart=1./ncat;/*set category probs equal*/
    for(i=0;i<ncat;i++){/*set root probs equal*/
      rootp[i][0]=0.5;
      rootp[i][1]=0.5;
      rhop[i]=rhostart;
    }
    for(i=0;i<maxQ;i++){/*set gene gain and loss rates equal*/
      Qset[i][0][0]=-1.;
      Qset[i][0][1]=1.;
      Qset[i][1][0]=1.;
      Qset[i][1][1]=-1.;
    }
    *alpha=1.;
  }
}/*initparms*/

/*calculate category mixing probs from major category probs*/
void rhofromrhop(double *rho, double *rhop, int gammacat, int nmajor)
{
  int i,j;
  double gprob;
  
  gprob=1./gammacat;
  for(i=0;i<nmajor;i++)
    for(j=0;j<gammacat;j++)
      rho[i*gammacat+j]=rhop[i]*gprob;
}/*rhofromrhop*/

/*calculate category root probs from major category probs*/
void rootfromrootp(double **rootset, double **rootp, int gammacat,int nmajor,int nstate)
{
  int i,j,k;

  for(i=0;i<nmajor;i++)
    for(j=0;j<gammacat;j++)
      for(k=0;k<nstate;k++)
	rootset[i*gammacat+j][k]=rootp[i][k];
}/*rootfromrootp*/

/*work out rate multipliers for each category*/
void set_rmult(double *rmult, double alpha, int gammacat, int nmajor)
{
  int i,j;
  
  for(i=0;i<nmajor;i++)
    for(j=0;j<gammacat;j++)
      rmult[i*gammacat+j]=gamma_rate(alpha,gammacat,j);
}/*set_rmult*/

/*return the rate for the ith of k discrete gamma categories with shape parameter alpha*/
double gamma_rate(double alpha, int k, int i)
{
  double a,b,invbeta,ri;

  invbeta=1./alpha;/*gives mean rate 1 over all categories*/
  a=gsl_cdf_gamma_Pinv((double)i/k,alpha,invbeta);/*lower cutpoint for category i from inverse gamma cdf*/
  b=gsl_cdf_gamma_Pinv((double)(i+1)/k,alpha,invbeta);/*upper cutpoint for category i from inverse gamma cdf*/
  if(i<(k-1)) ri=(gsl_sf_gamma_inc_P(alpha+1,b*alpha)-gsl_sf_gamma_inc_P(alpha+1,a*alpha))*k;/*get the mean rate for category i*/
  else ri=(1.-gsl_sf_gamma_inc_P(alpha+1,a*alpha))*k;/*get the mean rate for category i=k*/
  return ri;
}/*gamma_rate*/

/*print the set of rate matrices*/
void print_Qset(double ***Qset, int nQ, int nstate,FILE *fv)
{
  int i,j,k;
  for(i=0;i<nQ;i++){
    fprintf(fv,"Rate matrix %d:\n",i);
    for(j=0;j<nstate;j++){
      for(k=0;k<nstate;k++){
	fprintf(fv,"%.12f ",Qset[i][j][k]);
      }
      fprintf(fv,"\n");
    }
    fprintf(fv,"\n");
  }
}/*print_Qset*/

/*get P matrix from 2 x 2 rate matrix*/
/*i is index of the rate matrix to use over time t. rmult is a vector of rate multipliers for each category, and c is the index of the category we need. edgescale is a multiplier that applies to all edge lengths and all rate categories.*/
void PfromQ22(double **P, double ***Qset, int i, double t, double *rmult, int c, double *edgescale)
{
  double rsum,a,b,rt;
  
  rsum=Qset[i][0][1]+Qset[i][1][0];/*could precalculate this because it doesn't depend on t*/
  a=Qset[i][0][1]/rsum;/*could precalculate this*/
  rt=t*rmult[c]* (*edgescale);/*edge length * rate for this category * edge scale factor*/
  b=exp(-rsum*rt);
  P[0][0]=1-a+b*a;
  P[0][1]=1-P[0][0];
  P[1][1]=a+b*(1-a);
  P[1][0]=1-P[1][1];
}/*PfromQ22*/

/*print the transition probability matrix*/
void print_P(double **P, int nstate, double t)
{

  int j,k;
      printf("Transition probability matrix for time %f:\n",t);
    for(j=0;j<nstate;j++){
      for(k=0;k<nstate;k++) printf("%f ",P[j][k]);
      printf("\n");
    }
}/*print_P*/

/*test PfromQ22 by printing out P matrix for a range of times*/
void testPfromQ22(double **P, double ***Qset, double *rmult, double *edgescale)
{
  double t;
  int s;

  for(s=0;s<10;s++){
    t=1.*s;
    PfromQ22(P,Qset,0,t,rmult,0, edgescale);
    print_P(P,NSTATE,t);
  }
}/*testPfromQ22*/

/*calculate stationary probabilities for a specified 2x2 rate matrix (index i from Qset)*/
void stationaryprob22(double **piset, double ***Qset, int i)
{
  double rsum;

  rsum=Qset[i][0][1]+Qset[i][1][0];
  piset[i][0]=Qset[i][1][0]/rsum;
  piset[i][1]=Qset[i][0][1]/rsum;
}/*stationaryprob22*/

/*print the stationary probabilities for a specified rate matrix (index i)*/
void print_pi(double **piset, int nstate, int i)
{
  int j;
      printf("Stationary probabilities\n");
    for(j=0;j<nstate;j++) printf("%f ",piset[i][j]);
      printf("\n");
}/*print_pi*/


/*rescale a rate matrix so the expected number of events per unit time at equilibrium is 1*/
void Q_scale(double ***Qset, double **piset, int i, int nstate)
{
  int j,k;
  double dsum;
  
  dsum=0.;
  for(j=0;j<nstate;j++){
    dsum-=Qset[i][j][j]*piset[i][j];
  }
  for(j=0;j<nstate;j++){
    for(k=0;k<nstate;k++) Qset[i][j][k]=Qset[i][j][k]/dsum;
  }
}/*Q_scale*/

/*recursive calculation of lower conditional site likelihoods on tree*/
/*i is index of rate category to use*/
void treelike(TTree *tree, TNode *node, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, int i, double *rmult,double *edgescale)
{
  int r,s,n;
  double leftlike, rightlike;

  if(node->branch1!=NULL) treelike(tree,node->branch1, ngenes, nstates, Qset, Pleft, Pright, i,rmult,edgescale);/*visit left subtree*/
  if(node->branch2!=NULL) treelike(tree,node->branch2, ngenes, nstates, Qset, Pleft, Pright, i,rmult,edgescale);/*visit right subtree*/

  /*now do the calculation at this vertex*/
  if(node->tipNo==-1){/*not a leaf (which will already have conditional likelihoods)*/
    PfromQ22(Pleft,Qset,node->Qind1[i],node->length1,rmult,i,edgescale);/*left edge transition prob*/
    PfromQ22(Pright,Qset,node->Qind2[i],node->length2,rmult,i,edgescale);/*right edge transition prob*/

    /*printf("Left rate matrix %d\n",node->Qind1[i]);
    printf("Right rate matrix %d\n",node->Qind2[i]);
    print_P(Pleft,nstates,node->length1);
    print_P(Pright,nstates,node->length2);*/
    /*  printf("Lower conditional likelihoods for left subtree\n");
    for(s=0;s<nstates;s++){
      for(n=0;n<ngenes;n++) printf("%f ",node->branch1->CLike[i][s][n]);
      printf("\n");
    }
    printf("\n");

    printf("Lower conditional likelihoods for right subtree\n");
    for(s=0;s<nstates;s++){
      for(n=0;n<ngenes;n++) printf("%f ",node->branch2->CLike[i][s][n]);
      printf("\n");
    }
    printf("\n");*/

    for(n=0;n<ngenes;n++){/*all gene families*/
      for(s=0;s<nstates;s++){/*all states at vertex*/
	leftlike=0.;
	rightlike=0.;
	for(r=0;r<nstates;r++){/*all states in descendant*/
	  leftlike+=node->branch1->CLike[i][r][n]*Pleft[s][r];/*sum over states at left subtree*/
	  rightlike+=node->branch2->CLike[i][r][n]*Pright[s][r];/*sum over states at right subtree*/
	}
	node->CLike[i][s][n]=leftlike*rightlike;
	node->branch1->ULike[i][s][n]=leftlike;/*these are contributions to ULike which we'll need on the way up*/
	node->branch2->ULike[i][s][n]=rightlike;
      }
    }
  /*printf("Conditional likelihoods\n");
    for(s=0;s<nstates;s++){
      for(n=0;n<ngenes;n++) printf("%f ",node->CLike[s][n]);
      printf("\n");
    }
    printf("\n");*/
  }
  /*else{
    printf("visiting leaf %s\n",tree->names[node->tipNo]);
  }*/

}/*treelike*/

void get_treelike_cat(TTree *tree, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double **rootset, int c, double *rmult, double *edgescale)/*calculate site likelihoods for tree and given category c*/
     /*assumes we've already calculated scaled rate matrices, allocated memory for transition probabilities, and calculated stationary probabilities*/
     /*sets the gene family log likelihoods and sums them to get the total log likelihood*/
{
  int j,k;

  treelike(tree, tree->root, ngenes, nstates, Qset, Pleft, Pright, c,rmult,edgescale);/*calculate likelihood on tree, category c*/
  /*printf("Root conditional likelihoods\n");
    for(j=0;j<nstates;j++){
      for(k=0;k<ngenes;k++) printf("%f ",tree->root->CLike[j][k]);
      printf("\n");
    }
    printf("\n");*/
  for(j=0;j<ngenes;j++){
      tree->likes[c][j]=0.;
      for(k=0;k<nstates;k++)
	tree->likes[c][j]+=rootset[c][k]*tree->root->CLike[c][k][j];
  }


  /*for(k=0;k<nstates;k++) printf("Category %d, state %d, root prob %f\n",c,k,rootset[c][k]);*/

}/*get_treelike_cat*/

/*Get log likelihood at a vertex in preorder, and do edge length optimization. Must have been preceded by a postorder traversal, and initialization of upper subtree likelihoods at root. LRind is 1 for left edge, 2 for right edge. The log likelihood returned is the one we got BEFORE optimizing any edges.*/
double vertexlike(TTree *tree, TNode *node, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double *rho, genomedata *gd, double **rootset,double *rmult, double *edgescale)
{
  double treeloglike=0.;
  int c;

  if(node->tipNo==-1){
    /*get likelihood using left edge*/
    treeloglike=vertexlike_subtrees(tree, node, ncat, ngenes, nstates, Qset, Pleft, Pright, Panc, rho, gd, rootset, 1, rmult,edgescale);
    /*printf("left edge treeloglike=%f\n",treeloglike);*/
    if(node->optflag){/*optimize left edge*/
      edge_opt(tree,node,treeloglike,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,rmult,edgescale,gd,rootset,1);
      update_branchULike(node,ngenes,nstates,ncat,Qset,Pleft,Pright,1,rmult,edgescale);/*update the part of the upper subtree likelihood for the other edge that we changed*/
    }
 
    /*get likelihood using right edge*/
    treeloglike=vertexlike_subtrees(tree, node, ncat, ngenes, nstates, Qset, Pleft, Pright, Panc, rho, gd, rootset, 2, rmult,edgescale);
    /*printf("right edge treeloglike=%f\n",treeloglike);*/
    if(node->optflag){/*optimize right edge*/
      edge_opt(tree,node,treeloglike,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,rmult,edgescale,gd,rootset,2);
      update_branchULike(node,ngenes,nstates,ncat,Qset,Pleft,Pright,2,rmult,edgescale);
     }
    if((int)node!=(int)tree->root) update_ULike(node, ngenes, nstates, ncat);/*update node->branch1->ULike and node->branch2->ULike if not at root*/
    /*if((int)node==(int)tree->root)*/ /*print_UCLike(node,ngenes,nstates,ncat);*//*print lower and upper subtree likelihoods*/
  
    /*recursive calls to do preorder: because we'd need to update upper subtree likelihoods everywhere after optimizing an edge, we stop if we've optimized at this vertex*/
    if(node->branch1!=NULL && (!node->optflag)){/*not a leaf*/
      for(c=0;c<ncat;c++) PfromQ22(Panc[c],Qset,node->Qind1[c],node->length1,rmult,c,edgescale);/*left edge transition prob*/
      vertexlike(tree, node->branch1, ncat, ngenes, nstates, Qset, Pleft, Pright, Panc, rho, gd, rootset,rmult,edgescale);
    }
    if(node->branch2!=NULL && (!node->optflag)){
      for(c=0;c<ncat;c++) PfromQ22(Panc[c],Qset,node->Qind2[c],node->length2,rmult,c,edgescale);/*right edge transition prob*/
      vertexlike(tree, node->branch2, ncat, ngenes, nstates, Qset, Pleft, Pright, Panc, rho, gd, rootset,rmult,edgescale);
    }

  }
  /*else printf("leaf %s\n",tree->names[node->tipNo]);*/
  return(treeloglike);
}/*vertexlike*/

/*VERSION BEFORE 10/7/08: THINK THIS IS WRONG. NEEDS TO REGENERATE P MATRICES FOR EACH CATEGORY*/
/*update the part of upper subtree likelihood that changed after we optimized an edge length leading out of this vertex. LRind is 1 if we optimized left edge, 2 if right.*/
/* void update_branchULike(TNode *node,int ngenes,int nstates,int ncat,double **Pleft,double **Pright,int LRind) */
/* { */

/*   int i,n,s,r; */
/*   double like; */
  
/*   for(i=0;i<ncat;i++){/\*all categories*\/ */
/*     for(n=0;n<ngenes;n++){/\*all gene families*\/ */
/*       for(s=0;s<nstates;s++){/\*all states at vertex*\/ */
/* 	like=0.; */
/* 	for(r=0;r<nstates;r++){/\*all states in descendant*\/ */
/* 	  if(LRind==1) like+=node->branch1->CLike[i][r][n]*Pleft[s][r];/\*sum over states at left subtree*\/ */
/*  	  else like+=node->branch2->CLike[i][r][n]*Pright[s][r];/\*sum over states at right subtree*\/ */
/*  	} */
/*  	if(LRind==1) node->branch1->ULike[i][s][n]=like;  */
/*  	else node->branch2->ULike[i][s][n]=like;  */
/*       } */
/*     } */
/*   } */
/* }/\*update_branchULike*\/ */

/*update the part of upper subtree likelihood that changed after we optimized an edge length leading out of this vertex. LRind is 1 if we optimized left edge, 2 if right.*/
void update_branchULike(TNode *node,int ngenes,int nstates,int ncat,double ***Qset, double **Pleft,double **Pright, int LRind, double *rmult, double *edgescale)
{

  int i,n,s,r;
  double like;
  
  for(i=0;i<ncat;i++){/*all categories*/
    if(LRind==1) PfromQ22(Pleft,Qset,node->Qind1[i],node->length1,rmult,i,edgescale);/*left edge transition prob*/
    else PfromQ22(Pright,Qset,node->Qind2[i],node->length2,rmult,i,edgescale);/*right edge transition prob*/
    for(n=0;n<ngenes;n++){/*all gene families*/
      for(s=0;s<nstates;s++){/*all states at vertex*/
	like=0.;
	for(r=0;r<nstates;r++){/*all states in descendant*/
	  if(LRind==1) like+=node->branch1->CLike[i][r][n]*Pleft[s][r];/*sum over states at left subtree*/
 	  else like+=node->branch2->CLike[i][r][n]*Pright[s][r];/*sum over states at right subtree*/
 	}
 	if(LRind==1) node->branch1->ULike[i][s][n]=like; 
 	else node->branch2->ULike[i][s][n]=like; 
      }
    }
  }
}/*update_branchULike*/



/*calculate likelihood at a vertex using upper and lower subtree likelihoods, LRind=1 for left edge, 2 for right edge. Sets tree->likes and changes node->branch1->nULike (if LRind=1) or node->branch2->nULike (if LRind=2)*/
double vertexlike_subtrees(TTree *tree, TNode *node, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double *rho, genomedata *gd, double **rootset, int LRind, double *rmult, double *edgescale)
{
  int c,j,r,s;
  double treeloglike,ulike,bsum,blike;

  if((int)node==(int)tree->root) treeloglike=vertexlike_subtrees_root(tree,node,ncat,ngenes,nstates,Qset,Pleft,Pright,rho,gd,rootset,LRind,rmult,edgescale);/*initialize at root*/

  else{/*not at root*/
    for(j=0;j<ngenes;j++) tree->likes[ncat][j]=0.;
    treeloglike=0.;
    for(c=0;c<ncat;c++){/*each category*/
      if(LRind==1){
	PfromQ22(Pleft,Qset,node->Qind1[c],node->length1,rmult,c,edgescale);/*left edge transition prob*/
	/*print_P(Pleft,2,node->length1);*/
	/*printf("edge length %f\n",node->length1);*/
      }
      else{
	PfromQ22(Pright,Qset,node->Qind2[c],node->length2,rmult,c,edgescale);/*right edge transition prob*/
	/*print_P(Pright,2,node->length2);*/
	/*printf("edge length %f\n",node->length2);*/
      }
      /*print_P(Panc[c],2,node->length0);*/
      /*if(LRind==1) printf("edge %f\n",node->length1);
	else printf("edge %f\n",node->length2);*/
      for(j=0;j<ngenes;j++){/*each gene family*/
	ulike=0.;
	for(s=0;s<nstates;s++){/*each state at vertex*/
	  bsum=0.;
	  blike=0.;
	  for(r=0;r<nstates;r++){/*each state at descendant vertex*/
	    if(LRind==1){/*left edge*/
	      blike+=Panc[c][r][s]*node->ULike[c][r][j]*node->branch2->ULike[c][s][j];/*using the contribution to ULike stored on the way down the tree and the transition prob on the ancestral edge*/
	      bsum+=Pleft[s][r]*node->branch1->CLike[c][r][j];
	    }
	    else{/*right edge*/
	      blike+=Panc[c][r][s]*node->ULike[c][r][j]*node->branch1->ULike[c][s][j];
	      bsum+=Pright[s][r]*node->branch2->CLike[c][r][j];
	    }
	  }
 
	  /*printf("bsum %f blike %f\n",bsum,blike);*/

	  /*now sum over categories to get likelihood for this gene family*/
	  if(LRind==1){
	    node->branch1->nULike[c][s][j]=blike;/*left upper subtree likelihood*/
	    ulike+=bsum*node->branch1->nULike[c][s][j];
	  }
	  else{
	    node->branch2->nULike[c][s][j]=blike;/*right upper subtree likelihood*/
	    ulike+=bsum*node->branch2->nULike[c][s][j];
	  }
	}
	/*    printf("ulike %f\n",ulike);*/
	tree->likes[c][j]=ulike;
	tree->likes[ncat][j]+=tree->likes[c][j]*rho[c];
      }
    }
/*     for(j=0;j<ngenes;j++){/\*sum over gene families*\/ */
/*       tree->likes[ncat][j]=log(tree->likes[ncat][j]); */
/*       treeloglike+=tree->likes[ncat][j]*gd->weights[j]; */
/*     } */
  treeloglike=cond_like(tree,gd,ncat);
  }
  /*printf("treeloglike=%f\n",treeloglike);*/
  return treeloglike;
}/*vertexlike_subtrees*/

/*initialize upper subtree likelihoods at root and return log likelihood*/
double vertexlike_subtrees_root(TTree *tree, TNode *node, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double *rho, genomedata *gd, double **rootset, int LRind, double *rmult, double *edgescale)
{
  int c,j,r,s;
  double treeloglike,ulikel,uliker,leftlike,rightlike;

  /*printf("root\n");*/
  for(j=0;j<ngenes;j++) tree->likes[ncat][j]=0.;
  treeloglike=0.;

  for(c=0;c<ncat;c++){/*sum over categories*/
    PfromQ22(Pleft,Qset,tree->root->Qind1[c],tree->root->length1,rmult,c,edgescale);/*left edge transition prob used in right upper subtree likelihood*/
    PfromQ22(Pright,Qset,tree->root->Qind2[c],tree->root->length2,rmult,c,edgescale);/*right edge transition prob*/
    for(j=0;j<ngenes;j++){
      ulikel=0.;
      uliker=0.;
      for(s=0;s<nstates;s++){
	node->branch1->ULike[c][s][j]=rootset[c][s];/*probability of state h at root in category c*/
	node->branch2->ULike[c][s][j]=rootset[c][s];
	leftlike=0.;
	rightlike=0.;
	for(r=0;r<nstates;r++){
	  leftlike+=Pright[s][r]*node->branch2->CLike[c][r][j];/*LEFT upper subtree likelihood at root*/
	  rightlike+=Pleft[s][r]*node->branch1->CLike[c][r][j];/*RIGHT upper subtree likelihood at root*/
	}
	node->branch1->ULike[c][s][j]*=leftlike;
	node->branch2->ULike[c][s][j]*=rightlike;
	ulikel+=rightlike*node->branch1->ULike[c][s][j];/*we don't need to calculate on both sides: both the same*/
	uliker+=leftlike*node->branch2->ULike[c][s][j];
      }
      if(LRind==1) tree->likes[c][j]=uliker;
      else tree->likes[c][j]=ulikel;
      tree->likes[ncat][j]+=tree->likes[c][j]*rho[c];
    }
  }
/*   for(j=0;j<ngenes;j++){/\*sum over gene families*\/ */
/*     tree->likes[ncat][j]=log(tree->likes[ncat][j]); */
/*     treeloglike+=tree->likes[ncat][j]*gd->weights[j]; */
/*   } */
  treeloglike=cond_like(tree,gd,ncat);
  return(treeloglike);
}/*vertexlike_subtrees_root*/

/*get gene family conditional likelihoods, and sum logs of these to get the conditional tree log likelihood. Assumes gd contains the set of unique patterns and the set of unobservable patterns (immediately following the unique patterns), and that gd->nunique and gd->nhidden are the numbers of each of these)*/
double cond_like(TTree *tree,genomedata*gd,int ncat)
{
  int c,j;
  double Lplus=1.,treeloglike,lplus=0.;

  /*TEST VARIABLES*/
  /*int i;
    double test=0.,Lplusc,testsum=0.;*/
  /***************/

  if(gd->nhidden>0){
    Lplus=1.;
    for(j=gd->nunique;j<gd->nunique+gd->nhidden;j++) Lplus-=tree->likes[ncat][j];/*sum likelihoods of unobservable events*/
    lplus=log(Lplus);
  }
  treeloglike=0.;
  for(j=0;j<gd->nunique;j++){/*sum over gene families*/

    /*TEST CODE: THE OTHER WAY OF DOING THE COMPUTATION: I think this may be what Brian is doing*/
/*     test=0.; */
/*     for(c=0;c<ncat;c++){ */
/*       Lplusc=1.; */
/*       /\*get Li,c^minus*\/ */
/*       for(i=gd->nunique;i<gd->nunique+gd->nhidden;i++) Lplusc-=tree->likes[c][i]; */
/*       test+=tree->likes[c][j]/(Lplusc*ncat);/\*divide category likelihood by unobservable category likelihood*\/ */
/*     } */
/*     testsum+=log(test)*gd->weights[j]; */

    /*****************************/



    if (gd->nhidden>0){
      for(c=0;c<ncat;c++) tree->likes[c][j]/=Lplus;/*likelihood for gene family conditional on category and observability*/
      tree->likes[ncat][j]=log(tree->likes[ncat][j])-lplus;/*marginal log likelihood for gene family, conditional on observability*/
    }
    else tree->likes[ncat][j]=log(tree->likes[ncat][j]);
    treeloglike+=tree->likes[ncat][j]*gd->weights[j];
  }
  /*  printf("eq 3 l=%f,eq 4 l=%f\n",treeloglike,testsum);*/
  return(treeloglike);
}/*cond_like*/

/*update upper subtree likelihood after call to vertexlike_subtrees*/
void update_ULike(TNode *node, int ngenes, int nstates, int ncat)
{
  int c,s,j;
  for(c=0;c<ncat;c++){
    for(s=0;s<nstates;s++){
      for(j=0;j<ngenes;j++){
	node->branch1->ULike[c][s][j]=node->branch1->nULike[c][s][j];
	node->branch2->ULike[c][s][j]=node->branch2->nULike[c][s][j];
      }
    }
  }
}/*update_ULike*/

/*print out upper and lower subtree likelihoods at a vertex*/
void print_UCLike(TNode *node, int ngenes, int nstates, int ncat)
{
  int c,r,j;

  printf("This vertex upper subtree likelihoods\n");
  for(c=0;c<ncat;c++){
    printf("Category %d\n",c);
    for(r=0;r<nstates;r++){
      for(j=0;j<ngenes;j++){
	printf("%f ",node->ULike[c][r][j]);
      }
      printf("\n");
    }
    printf("\n");
  }

  printf("This vertex lower subtree likelihoods\n");
  for(c=0;c<ncat;c++){
    printf("Category %d\n",c);
    for(r=0;r<nstates;r++){
      for(j=0;j<ngenes;j++){
	printf("%f ",node->branch1->CLike[c][r][j]);
      }
      printf("\n");
    }
    printf("\n");
  }

  if(node->branch1!=NULL){
    printf("Left upper subtree likelihoods\n");
    for(c=0;c<ncat;c++){
      printf("Category %d\n",c);
      for(r=0;r<nstates;r++){
	for(j=0;j<ngenes;j++){
	  printf("%f ",node->branch1->ULike[c][r][j]);
	}
	printf("\n");
      }
      printf("\n");
    }

    printf("Left lower subtree likelihoods\n");
    for(c=0;c<ncat;c++){
      printf("Category %d\n",c);
      for(r=0;r<nstates;r++){
	for(j=0;j<ngenes;j++){
	  printf("%f ",node->branch1->CLike[c][r][j]);
	}
	printf("\n");
      }
      printf("\n");
    }
  }
  else printf("No left subtree\n");

  if(node->branch2!=NULL){
    printf("Right upper subtree likelihoods\n");
    for(c=0;c<ncat;c++){
      printf("Category %d\n",c);
      for(r=0;r<nstates;r++){
	for(j=0;j<ngenes;j++){
	  printf("%f ",node->branch2->ULike[c][r][j]);
	}
	printf("\n");
      }
      printf("\n");
    }

    printf("Right lower subtree likelihoods\n");
    for(c=0;c<ncat;c++){
      printf("Category %d\n",c);
      for(r=0;r<nstates;r++){
	for(j=0;j<ngenes;j++){
	  printf("%f ",node->branch2->CLike[c][r][j]);
	}
	printf("\n");
      }
      printf("\n");
    }
  }
  else printf("No right subtree\n");
}/*print_UCLike*/


/*get tree log likelihood: if preorder nonzero, do a preorder traversal after postorder. Otherwise, just do postorder*/
double get_treelike(TTree *tree, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double **rootset, genomedata *gd, double *rho,double *rmult,double *edgescale,int preorder)
{
  int i,k;
  double treeloglike;

  if(tree->rooted){
    /*printf("postorder\n");*/
    for(i=0;i<ncat;i++){
      get_treelike_cat(tree, ngenes, nstates, Qset, Pleft, Pright, rootset, i,rmult,edgescale);/*postorder to get lower conditional likeilhoods*/
    }

    if(!preorder){/*postorder only*/
      /*    printf("Gene family log likelihoods\n");*/
      treeloglike=0.;
      for(k=0;k<ngenes;k++){/*sum over categories at site*/
	tree->likes[ncat][k]=tree->likes[0][k]*rho[0];
	for(i=1;i<ncat;i++) tree->likes[ncat][k]+=tree->likes[i][k]*rho[i];
	/*tree->likes[ncat][k]=log(tree->likes[ncat][k]);*/
	/*printf("%f ",(tree->likes[ncat][k]));*/
	/*treeloglike+=tree->likes[ncat][k]*gd->weights[k];*/
      }
      treeloglike=cond_like(tree,gd,ncat);
      /*printf("log likelihood %f\n",treeloglike);*/
    }
    else treeloglike=vertexlike(tree, tree->root, ncat, ngenes, nstates, Qset, Pleft, Pright, Panc, rho, gd,rootset,rmult,edgescale);/*preorder to get upper conditional likelihoods*/

    return(treeloglike);
  }
  else{
    printf("Tree must be rooted: exiting.\n");    
    exit(EXIT_FAILURE);
  }
}/*get_treelike*/

/*write results to file*/
void write_results(FILE *out_fv,TTree *tree, double treeloglike, parameters *params,int ncat,genomedata *gd, double *rho, double etime,double ewall, int conv,int optit,int notobs,int ngenes,int nmajor, int gammacat)
{
  int i,k;

  fprintf(out_fv,"\n");
  switch(params->ctype){/*observability*/
  case 0:fprintf(out_fv,"All gene family patterns assumed observable.\n");
    break;
  case 1:fprintf(out_fv,"Gene families present in at least one genome assumed observable.\n");
    break;
  case 2:
  case 3:
    fprintf(out_fv,"Gene families present in at least %d genomes assumed observable.\n",params->ctype);
    break;
  }
  if(notobs>0) fprintf(out_fv,"Warning: %d gene families are not observable according to this criterion. They will be given zero weight in likelihood calculations.\n",notobs);   
  fprintf(out_fv,"Read %d genomes, %d gene families.\n",gd->ntaxa,ngenes);
  if((params->edgeopt)||(params->parmopt)){
    fprintf(out_fv,"\nFinal log likelihood %.12f.\n",treeloglike);
    if(conv==CONVERGED) fprintf(out_fv,"Converged in %d iterations.\n",optit);
    else if(conv==EXCEEDED) fprintf(out_fv,"Did not converge in %d iterations.\n",optit);
  }
  else fprintf(out_fv,"\nLog likelihood %.12f.\n",treeloglike);
  fprintf(out_fv,"Wall clock time %.1f seconds. Processor time %.1f seconds (warning: this will wrap for long processes).\n",ewall,etime);
  if(params->parmopt){
    if(params->optchoice==QNEWT) fprintf(out_fv,"Using a GSL quasi-Newton optimizer for parameters other than edge lengths.\n");
    else if(params->optchoice==SIMPLEX) fprintf(out_fv,"Using a GSL simplex optimizer for parameters other than edge lengths.\n");
  }
  else fprintf(out_fv,"Parameters other than edge lengths treated as known.\n");
  
  if(params->edgeopt){
    fprintf(out_fv,"\nEstimated edge lengths using golden section:\n");
    copyedges(tree->root);/*put the optimized edges in the right places*/
    WriteTree(out_fv,tree);
  }
  else fprintf(out_fv,"Edge lengths treated as known.\n");
  if(params->edgescale!=1.) fprintf(out_fv,"All edge lengths multiplied by scale factor %.12f.\n",params->edgescale);
  if(params->printlikes){
    fprintf(out_fv,"\nGene family likelihoods:\n");
    for(i=0;i<ncat;i++){
      fprintf(out_fv,"Category %d\n",i);
      for(k=0;k<gd->ngenes;k++) fprintf(out_fv,"%.12f ",(tree->likes[i][gd->match[k]]));
      fprintf(out_fv,"\n");
    }
    fprintf(out_fv,"\nMarginal gene family log likelihoods:\n");
    for(k=0;k<gd->ngenes;k++) fprintf(out_fv,"%.12f ",(tree->likes[ncat][gd->match[k]]));
    fprintf(out_fv,"\n\nObservability (1 for observable, 0 for not, under -c%d). Likelihoods and posterior probabilities are conditioned on observability, and are therefore not meaningful for unobservable gene families.\n",params->ctype);
    for(k=0;k<gd->ngenes;k++) fprintf(out_fv,"%d ",(gd->weights[gd->match[k]]>0));
				     
    if(ncat>1) write_pprobs(out_fv,gd,tree->likes,rho,ncat,nmajor,gammacat);/*write posterior probabilities*/

  }
}/*write_results*/

/*calculate and write posterior probabilities*/
void write_pprobs(FILE *out_fv,genomedata *gd,double **likes, double *rho,int ncat,int nmajor,int gammacat)
{
  double **pprobs;
  double mpp;
  int *c;
  int h,i,j,k;

  if((pprobs=(double **)calloc(ncat+1,sizeof(double*)))==NULL){
    (void)fprintf(stderr,"ERROR: calloc failed in write_pprobs.\n");
    (void)exit(EXIT_FAILURE);
  }
  for(i=0;i<ncat+1;i++) if((pprobs[i]=(double*)calloc(gd->ngenes,sizeof(double)))==NULL){
    (void)fprintf(stderr,"ERROR: calloc failed in write_pprobs.\n");
    (void)exit(EXIT_FAILURE);
  }

  if((c=(int*)calloc(gd->ngenes,sizeof(int)))==NULL){
    (void)fprintf(stderr,"ERROR: calloc failed in write_pprobs.\n");
    (void)exit(EXIT_FAILURE);
  }

  for(j=0;j<gd->ngenes;j++){
    for(i=0;i<ncat;i++) pprobs[ncat][j]+=likes[i][gd->match[j]]*rho[i];/*probability of data for this gene family: backtransforming the stored gene family log likelihood gives too large rounding error here*/
    mpp=0.;
    for(i=0;i<ncat;i++){
      pprobs[i][j]=likes[i][gd->match[j]]*rho[i]/pprobs[ncat][j];/*posterior probability of this category*/
      if(pprobs[i][j]>mpp){
	mpp=pprobs[i][j];
	c[j]=i;
      }
    }
  }

  fprintf(out_fv,"\n");
  fprintf(out_fv,"\nEmpirical Bayes posterior probabilities:\n");
  
  for(i=0;i<nmajor;i++)
    for(j=0;j<gammacat;j++){
      k=gammacat*i+j;
      fprintf(out_fv,"Category %d (major category %d, rate class %d)\n",k,i,j);
      for(h=0;h<gd->ngenes;h++) fprintf(out_fv,"%f ",pprobs[k][h]);
      fprintf(out_fv,"\n");      
  }
  fprintf(out_fv,"\n");
  fprintf(out_fv,"Maximum posterior probability category:\n");
  for(j=0;j<gd->ngenes;j++) fprintf(out_fv,"%d ",c[j]);
  fprintf(out_fv,"\n\n");

  for(i=0;i<ncat+1;i++) free(pprobs[i]);
  free(pprobs);
  free(c);
}/*write_pprobs*/

/*write out parameters (initial or estimated)*/
void write_parms(FILE *out_fv,double ***Qset,double *rhop,double **rootp,int nmajor,int gammacat,int maxQ,double alpha)
{
  int i;
  print_Qset(Qset,maxQ,NSTATE,out_fv);/*print out the rate matrices*/
  for(i=0;i<nmajor;i++) fprintf(out_fv,"Major category %d: mixing probability %.12f, root probability of state zero %.12f\n",i,rhop[i],rootp[i][0]);
  if(gammacat>1){
    fprintf(out_fv,"Gamma shape parameter alpha=%.12f\n",alpha);
    fprintf(out_fv,"Class\tRate\n");
    for(i=0;i<gammacat;i++) fprintf(out_fv,"%d\t%.12f\n",i,gamma_rate(alpha,gammacat,i));
  }
  else fprintf(out_fv,"Gamma shape parameter ignored: only one rate class.\n");
}/*write_parms*/

/*write settings to terminal*/
void write_settings(parameters *params,char*outfilename,char *pfilename)
{
  printf("%s version %s\n%s\n",PROGRAM_NAME,PROGRAM_VERSION,COPYRIGHT);
  switch(params->ctype){/*observability*/
  case 0:printf("All gene family patterns assumed observable.\n");
    break;
  case 1:printf("Gene families present in at least one genome assumed observable.\n");
    break;
  case 2:
  case 3:
    printf("Gene families present in at least %d genomes assumed observable.\n",params->ctype);
    break;
  }
  if(params->gammacat>1)printf("Gamma rate variation with %d rate classes.\n",params->gammacat);
  else printf("No gamma rate variation.\n");
  if(params->edgeopt) printf("Edge lengths will be estimated.\n");
  else printf("Edge lengths will be treated as fixed.\n");
  if(params->edgescale!=1.) printf("All edge lengths multiplied by scale factor %.12f.\n",params->edgescale);
  if(params->parmopt){
    if(params->optchoice==QNEWT) printf("Other parameters will be estimated using quasi-Newton.\n");
    else if(params->optchoice==SIMPLEX) printf("Other parameters will be estimated using simplex.\n");
  }
  else printf("Other parameters will be treated as fixed.\n");
  if(params->printlikes) printf("Gene family likelihoods and empirical Bayes posterior probabilities will be printed to %s.\n",outfilename);
  if(params->startparms) printf("Initial parameter estimates will be read from %s.\n",pfilename);
}/*write_settings*/

/*write simulated data to output file*/
void write_simdata(FILE *out_fv,int numTaxa,int sgf,TTree *tree)
{
  int i,j;

  fprintf(out_fv,"%d %d\n",numTaxa,sgf);
  for(j=0;j<numTaxa;j++){
    fprintf(out_fv,"%s\t",tree->names[j]);
    for(i=0;i<sgf;i++) fprintf(out_fv,"%d ",(int)tree->tips[j]->CLike[0][0][i]);
    fprintf(out_fv,"\n");
  }

  /*write root states and categories too*/
  fprintf(out_fv,"\nRoot states:\n");
  for(i=0;i<sgf;i++) fprintf(out_fv,"%d ",(int)tree->root->CLike[0][0][i]);
  fprintf(out_fv,"\nCategories:\n");
  for(i=0;i<sgf;i++) fprintf(out_fv,"%d ",(int)tree->root->CLike[1][0][i]);
  fprintf(out_fv,"\n");

}/*write_simdata*/

/*check whether a simulated gene family is observable*/
int check_observability(int numTaxa, TTree *tree, int i, parameters *params)
{
  int j,s=0;

  for(j=0;j<numTaxa;j++) s+=(int)tree->tips[j]->CLike[0][0][i];/*number of occurrences at leaves*/
  return(s>=params->ctype);/*is this gene family observable according to our setting?*/
}/*check_observability*/

/*sample category from a mixture with probabilities rho*/
int discrete_rand(double *rho, int ncat,gsl_rng *r)
{
  int i=0;
  double u,rhosum=0;

  u = gsl_rng_uniform (r);
  while(i<ncat){
    rhosum+=rho[i];
    if(u<rhosum) break;
    i++;
  }
  return(i);
}/*discrete_rand*/

/*generate category for gene family*/
void generate_categories(int ncat,double *rho,int sgf,double *catvec, gsl_rng *r, int i)
{
  /*for(i=0;i<sgf;i++) */
  catvec[i]=discrete_rand(rho,ncat,r);
}/*generate_categories*/

/*generate 0/1 root state for gene family*/
void generate_rootstate(double **rootset, int sgf,double *statevec,double *catvec,gsl_rng *r, int i)
{
  double p;

  /*for(i=0;i<sgf;i++){*/
    p=rootset[(int)catvec[i]][0];/*state probs for the category this gene family belongs to*/
    statevec[i]=(gsl_rng_uniform(r)>p);/*sample from those state probs*/
    /*printf("%d %d\n",(int)catvec[i],(int)statevec[i]);*//*these are the category and state at the root*/
    /*}*/
}/*generate_rootseq*/

/*generate states at all vertices, once root state generated*/
void generate_vertexstates(int sgf, double *statevec,double *catvec,TNode *node,gsl_rng *r,double **Pleft,double **Pright,double ***Qset,double *rmult,double *edgescale,int i)
{
  int c;
  double p;

  /*  for(i=0;i<sgf;i++){*//*for each gene family*/
    c=(int)catvec[i];/*category for this gene family*/
    if(node->branch1!=NULL){
      PfromQ22(Pleft,Qset,node->Qind1[c],node->length1,rmult,c,edgescale);/*generate left P matrix*/
      p=Pleft[(int)statevec[i]][0];/*probability of zero at descendant vertex given current state*/
      node->branch1->CLike[0][0][i]=(gsl_rng_uniform(r)>p);/*sample gene family at left vertex*/
    }
    if(node->branch2!=NULL){
      PfromQ22(Pright,Qset,node->Qind2[c],node->length2,rmult,c,edgescale);/*generate right P matrix*/
      p=Pright[(int)statevec[i]][0];/*probability of zero at descendant vertex given current state*/
      node->branch2->CLike[0][0][i]=(gsl_rng_uniform(r)>p);/*sample gene family at right vertex*/
    }
    /*}*/
  if(node->branch1!=NULL) generate_vertexstates(sgf,node->branch1->CLike[0][0],catvec,node->branch1,r,Pleft,Pright,Qset,rmult,edgescale,i);/*recurse to left vertex*/
  if(node->branch2!=NULL) generate_vertexstates(sgf,node->branch2->CLike[0][0],catvec,node->branch2,r,Pleft,Pright,Qset,rmult,edgescale,i);/*recurse to right vertex*/
}/*generate_vertexstates*/

/*generate simulated data on tree*/
void simulate_data(parameters *params, char *treefilename, char *outfilename, char *pfilename, int argc, char **argv)
{

  int numTrees,nmajor,ncat,maxQ=0,numTaxa,i;
  TTree **treeSet;
  FILE *tree_fv, *out_fv;
  double ***Qset;/*set of rate matrices*/
  double **Pleft, **Pright;/*transition probability matrix*/
  double **piset;/*set of stationary probabilities for each Q matrix*/
  double *rhop,*rho;/*mixing probabilities for categories*/
  double **rootp, **rootset;/*set of state probabilities at root for each category*/
  double *rmult;/*rate multipliers for each category*/
  double alpha;/*gamma shape parameter*/
  const gsl_rng_type * T;
  gsl_rng * r;

  if(!params->quiet) printf("Generating %d simulated gene families.\n",params->sgf);

  /*read the trees*/
    
  if ( (tree_fv=fopen(treefilename, "rt"))==NULL ) {/*read in the tree structure*/
    fprintf(stderr, "Error opening tree file: '%s'\n", treefilename);
    exit(0);
  }

  numTrees=CountTrees(tree_fv);
  treeSet=read_treefile(tree_fv,numTrees,params->sgf, NSTATE,params->gammacat);/*read in topology and rate matrix info*/
  if(numTrees>0) maxQ=setuprates(treeSet,numTrees,params->gammacat);/*copy rate matrix indices stored as edge lengths on treeSet[1] and upwards to base tree*/
  if(!params->quiet){
    printf("Read %d trees.\n",numTrees);
    printf("%d rate matrices are used.\n",maxQ);
  }
  nmajor=numTrees-1;/*major categories*/
  ncat=nmajor*params->gammacat;/*major categories * gamma rate classes*/
  numTaxa=treeSet[0]->numTips;/*number of taxa (need this to write the output file)*/
  Qset=make_Qset(maxQ,NSTATE);/*set up Q matrices*/
  piset=make_piset(maxQ,NSTATE);/*set up memory for the stationary probabilities*/
  rootset=make_piset(ncat,NSTATE);/*set up memory for root probabilities*/
  rootp=make_piset(nmajor,NSTATE);/*root probs for major categories*/
  Pleft=make_P(NSTATE);/*set up P matrix*/
  Pright=make_P(NSTATE);/*set up P matrix*/
  rho=make_rho(ncat);/*set up array of mixing probs*/
  rhop=make_rho(nmajor);/*parameters that determine mixing probs*/
  rmult=make_rho(ncat);/*vector of rate multipliers*/
  initparms(params,pfilename,Qset,rhop,rootp,nmajor,maxQ,&alpha);/*initialize parameters other than edges*/
  rhofromrhop(rho,rhop,params->gammacat,nmajor);/*cat prob=major cat prob * rate class prob*/
  /*print_rho(rho,rhop,params->gammacat,nmajor);*/
  rootfromrootp(rootset,rootp,params->gammacat,nmajor,NSTATE);/*root probabiility for each major cat * rate class*/
  set_rmult(rmult,alpha,params->gammacat,nmajor);/*rate multipliers for categories*/
  /*print_root(rootset,rootp,params->gammacat,nmajor,NSTATE);*/
  for(i=0;i<maxQ;i++){
    stationaryprob22(piset,Qset,i);/*stationary probabilities*/
    Q_scale(Qset,piset,i,NSTATE);/*scale Q matrix to one expected event per unit time at equilibrium*/
  }
  
  gsl_rng_env_setup();/*initialize rand*/

  T = gsl_rng_default;/*set up GSL pseudorandom generator*/
  r = gsl_rng_alloc (T);
  gsl_rng_set(r,time(NULL));/*seed with time*/

  /*note we're using CLike because it already exists, but cast everything from double to int*/
  i=0;
  while(i<params->sgf){/*generate one gene family at a time and reject if not observable*/
    generate_categories(ncat,rho,params->sgf,treeSet[0]->root->CLike[1][0],r,i);/*generate category*/
    generate_rootstate(rootset,params->sgf,treeSet[0]->root->CLike[0][0],treeSet[0]->root->CLike[1][0],r,i);/*generate root vertex*/
    generate_vertexstates(params->sgf, treeSet[0]->root->CLike[0][0],treeSet[0]->root->CLike[1][0],treeSet[0]->root,r,Pleft,Pright,Qset,rmult,&params->edgescale,i);/*preorder to generate data at each vertex*/
    i+=check_observability(numTaxa,treeSet[0],i,params);/*check for observability*/
  }


  /*open output file*/
  if ( (out_fv=fopen(outfilename, "wt"))==NULL ) {
    fprintf(stderr, "Error opening output file: '%s'\n", outfilename);
    exit(0);
  }



  write_simdata(out_fv,numTaxa,params->sgf,treeSet[0]);/*write to output file*/
  fprintf(out_fv,"\n%s version %s\n%s\n\nCommand:\n",PROGRAM_NAME,PROGRAM_VERSION,COPYRIGHT);
  for(i=0;i<argc;i++) fprintf(out_fv,"%s%s",argv[i],(i<argc-1) ? " " : "");/*write arguments to output file*/
  fprintf(out_fv,"\n");

  /*clean up*/
  fclose(tree_fv);
  fclose(out_fv);
  free_treeSet(treeSet,numTrees,NSTATE, numTrees,params->gammacat);
  free(outfilename);
  free(treefilename);
  free(pfilename);
  free_Qset(Qset,maxQ,NSTATE);
  free_P(Pleft,NSTATE);
  free_P(Pright,NSTATE);
  free_piset(piset,maxQ);
  free_piset(rootset,ncat);
  free_piset(rootp,nmajor);
  free(rho);
  free(rhop);
  free(rmult);
  free(params);
  gsl_rng_free (r);
}/*simulate_data*/

int main(int argc, char **argv)
{
  genomedata *gdn, *gd;/*first copy is what we read in, second copy is unique genes only*/
  parameters *params;
  char *infilename, *outfilename, *treefilename, *pfilename;
  TTree **treeSet;
  int conv=WORKING,i,optit=0,ncat,nmajor,numTrees,nhidden,notobs,ntotal,maxQ=0;
  FILE *tree_fv, *out_fv;
  double ***Qset;/*set of rate matrices*/
  double **Pleft, **Pright;/*transition probability matrix*/
  double ***Panc;/*set of transition probabilitiy matrices on edge as we preorder up the tree*/
  double **piset;/*set of stationary probabilities for each Q matrix*/
  double treeloglike,tlnew;/*log likelihood on tree*/
  double *rhop,*rho;/*mixing probabilities for categories*/
  double **rootp, **rootset;/*set of state probabilities at root for each category*/
  double *rmult;/*rate multipliers for each category*/
  double alpha;/*gamma shape parameter*/
  double etime,ewall;/*elapsed time*/
  clock_t starttime = clock();/*start time*/
  time_t startwall=time(NULL);/*start wall clock time*/



  /*allocate memory*/
  params=(parameters *)calloc(1,sizeof(parameters));
  infilename=(char *)calloc(LEN,sizeof(char));
  outfilename=(char *)calloc(LEN,sizeof(char));
  treefilename=(char *)calloc(LEN,sizeof(char));
  pfilename=(char *)calloc(LEN,sizeof(char));
  
  ReadParams(argc,argv,params,infilename,outfilename,treefilename,pfilename);/*read parameters*/
  if(params->printHelp){/*display help and exit*/
    printHelp();
    free(infilename);
    free(outfilename);
    free(treefilename);
    free(pfilename);
    free(params);
  }
  else if(params->simulate){/*generate simulated data and exit*/
    free(infilename);/*we don't need this when simulating*/
    simulate_data(params,treefilename,outfilename,pfilename,argc,argv);
  }
  else{/*do computations*/
    gdn=read_genomedata(infilename);/*read in the data*/
    nhidden=set_nhidden(gdn->ntaxa,params);/*how many unobservable patterns?*/
    gd=make_genomedata(gdn->ntaxa,gdn->ngenes,nhidden);/*generate unobservable patterns*/
    notobs=match_patterns(gdn,gd,params);/*reduce to unique patterns*/

    generate_hidden(gd,params);/*generate unobservable patterns*/
    ntotal=gd->nunique+nhidden;/*total number of patterns*/
    /*printgd(gdn);*/
    if(params->printdata) printgd(gd);/*print out the gene family data*/
    if(!params->quiet) write_settings(params,outfilename,pfilename);/*display settings?*/
    if((!params->quiet) && (notobs>0)) printf("Warning: %d gene families are not observable according to specified criteria. They will be given zero weight in likelihood calculations.\n",notobs);    
    if ( (tree_fv=fopen(treefilename, "rt"))==NULL ) {/*read in the tree structure*/
      fprintf(stderr, "Error opening tree file: '%s'\n", treefilename);
      exit(0);
    }

    numTrees=CountTrees(tree_fv);
    treeSet=read_treefile(tree_fv,numTrees,ntotal, NSTATE,params->gammacat);/*read in topology and rate matrix info*/
    if(numTrees>0) maxQ=setuprates(treeSet,numTrees,params->gammacat);/*copy rate matrix indices stored as edge lengths on treeSet[1] and upwards to base tree*/
    if(!params->quiet){
      printf("Read %d trees.\n",numTrees);
      printf("%d rate matrices are used.\n",maxQ);
      printf("Read %d genomes, %d gene families.\n",gd->ntaxa,gdn->ngenes);
    }
    nmajor=numTrees-1;/*major categories*/
    ncat=nmajor*params->gammacat;/*major categories * gamma rate classes*/

    fill_treedata(gd,treeSet,ncat);/*copy genome data to leaves*/
    Qset=make_Qset(maxQ,NSTATE);/*set up Q matrices*/
    piset=make_piset(maxQ,NSTATE);/*set up memory for the stationary probabilities*/
    rootset=make_piset(ncat,NSTATE);/*set up memory for root probabilities*/
    rootp=make_piset(nmajor,NSTATE);/*root probs for major categories*/
    Pleft=make_P(NSTATE);/*set up P matrix*/
    Pright=make_P(NSTATE);/*set up P matrix*/
    Panc=make_Panc(NSTATE,ncat);
    rho=make_rho(ncat);/*set up array of mixing probs*/
    rhop=make_rho(nmajor);/*parameters that determine mixing probs*/
    rmult=make_rho(ncat);/*vector of rate multipliers*/
    initparms(params,pfilename,Qset,rhop,rootp,nmajor,maxQ,&alpha);/*initialize parameters other than edges*/
    rhofromrhop(rho,rhop,params->gammacat,nmajor);/*cat prob=major cat prob * rate class prob*/
    /*print_rho(rho,rhop,params->gammacat,nmajor);*/
    rootfromrootp(rootset,rootp,params->gammacat,nmajor,NSTATE);/*root probabiility for each major cat * rate class*/
    set_rmult(rmult,alpha,params->gammacat,nmajor);/*rate multipliers for categories*/
    /*print_root(rootset,rootp,params->gammacat,nmajor,NSTATE);*/

    /*print_Qset(Qset,maxQ,NSTATE,stdout);*//*print out the starting values*/

    /*open output file*/
    if ( (out_fv=fopen(outfilename, "wt"))==NULL ) {
      fprintf(stderr, "Error opening output file: '%s'\n", outfilename);
      exit(0);
    }

    fprintf(out_fv,"%s version %s\n%s\n\nCommand:\n",PROGRAM_NAME,PROGRAM_VERSION,COPYRIGHT);
    for(i=0;i<argc;i++) fprintf(out_fv,"%s%s",argv[i],(i<argc-1) ? " " : "");/*write arguments to output file*/
    fprintf(out_fv,"\n\n");
    fprintf(out_fv,"Initial edge lengths:\n");
    WriteTree(out_fv,treeSet[0]);
    fprintf(out_fv,"\nRate matrix indices:\n");
    for(i=1;i<numTrees;i++){
      fprintf(out_fv,"Major category %d\n",i-1);
      WriteTree(out_fv,treeSet[i]);
    }
    fprintf(out_fv,"\nInitial parameter estimates\n");

#if defined(DEBUG) || defined(DEBUG2) || defined(DEBUG3) || defined(BRIAN_DEBUG) || defined(BRIAN_DEBUG2) || defined(BRIAN_DEBUG3)/*debugging code: see ML_genedata.h for what the outputs should be*/
    test_Qset(Qset);/*write some test values to rate matrices*/
    test_rootset(rootset);/*write some test values to root probabilities*/
    for(i=0;i<maxQ;i++) stationaryprob22(piset,Qset,i);/*stationary probabilities*/

#ifdef DEBUG/*for testinput.txt, test.tree*/
    rho[0]=0.5;
    rho[1]=0.5;
    for(i=0;i<maxQ;i++)Q_scale(Qset,piset,i,NSTATE);/*scale Q matrix to one expected event per unit time at equilibrium*/
#elif defined(DEBUG2)/*for testinput2.txt, test2.tree*/
    rho[0]=1.;
#elif defined(DEBUG3) /*for test_ML_fourtaxa_binary.phy, test_fourtaxa_c.tree*/
    rho[0]=1.;
#elif defined(BRIAN_DEBUG)/*for Bacillus.phy, Bacillus.tree*/
    rho[0]=1.;
#elif defined (BRIAN_DEBUG2)/*for Bacillus.phy, Bacillus_case2.tree*/
    rho[0]=1.;
#elif defined (BRIAN_DEBUG3)/*for Bacillus.phy, Bacillus_case2.tree*/
    rho[0]=1.;
#endif

#else/*normal code*/
    for(i=0;i<maxQ;i++){
      stationaryprob22(piset,Qset,i);/*stationary probabilities*/
      Q_scale(Qset,piset,i,NSTATE);/*scale Q matrix to one expected event per unit time at equilibrium*/
    }
#endif
    write_parms(out_fv,Qset,rhop,rootp,nmajor,params->gammacat,maxQ,alpha);/*initial parameter estimates*/
    treeloglike=get_treelike(treeSet[0],ncat,ntotal, NSTATE, Qset, Pleft, Pright, Panc, rootset, gd, rho,rmult,&params->edgescale,0);/*get log likelihood*/
    if(!params->quiet) printf("Initial log likelihood %.12f\n",treeloglike);

    tlnew=treeloglike;
    if((params->edgeopt)||(params->parmopt)){/*optimize edges/parameters*/
      conv=WORKING;
      optit=0;
      while(conv==WORKING){
	if(!params->quiet)printf("Iteration %d\n",optit+1);
	if(params->edgeopt){/*edge optimization*/
	  opt_edgeset(treeSet[0],treeSet[0]->root,ncat,ntotal,NSTATE,Qset,Pleft,Pright,Panc,rootset,gd,rho,rmult,&params->edgescale);/*do a sweep through the tree, optimizing pairs of descendant edges at each vertex in turn*/
	  tlnew=get_treelike(treeSet[0], ncat,ntotal, NSTATE, Qset, Pleft, Pright, Panc, rootset, gd, rho,rmult,&params->edgescale,0);/*get log likelihood: would be more efficient to get this directly from the preorder, but didn't figure out how to do that*/
	  if(!params->quiet)printf("\tOptimized edges, log likelihood %.12f\n",tlnew);
	}
	if(params->parmopt){/*other parameter optimization*/
	  tlnew=tree_opt(treeSet[0],ncat,nmajor,params->gammacat,ntotal,NSTATE,Qset,Pleft,Pright,Panc,rootp,rootset,gd,rhop,rho,rmult,&params->edgescale,&alpha,piset,params,maxQ);/*model parameter optimization*/
	  if(!params->quiet)printf("\tOptimized model parameters, log likelihood %.12f\n",tlnew);
	}
	if(fabs(treeloglike-tlnew)<OPTTOL){/*converged*/
	  conv=CONVERGED;
	  if(!params->quiet)printf("Converged.\n");
	}
	else if(optit>MAXOPTIT-2){/*reached max iterations*/
	  conv=EXCEEDED;
	  if(!params->quiet)printf("Reached max iterations.\n");
	}
	treeloglike=tlnew;
	optit++;
      }
    }
    

    etime=((double)clock() - starttime) / CLOCKS_PER_SEC;
    ewall=((double)time(NULL)-startwall);
    write_results(out_fv,treeSet[0],treeloglike,params,ncat,gd,rho,etime,ewall,conv,optit,notobs,gdn->ngenes,nmajor,params->gammacat);/*final edge lengths etc*/
    if(params->parmopt){
      fprintf(out_fv,"Final parameter estimates\n");
      write_parms(out_fv,Qset,rhop,rootp,nmajor,params->gammacat,maxQ,alpha);/*final parameter estimates*/
    }

    /*print out gradient and Hessian*/
    gradHessprint(treeSet[0],ncat,nmajor,params->gammacat,ntotal,NSTATE,Qset,Pleft,Pright,Panc,rootp,rootset,gd,rhop,rho,rmult,&params->edgescale,&alpha,piset,params,maxQ,out_fv,treeloglike);/*model parameter optimization*/
  
    /*clean up*/
    fclose(tree_fv);
    fclose(out_fv);
    free_treeSet(treeSet,numTrees,NSTATE, numTrees,params->gammacat);
    free_genomedata(gd);
    free_genomedata(gdn);
    free_Qset(Qset,maxQ,NSTATE);
    free_P(Pleft,NSTATE);
    free_P(Pright,NSTATE);
    free_Panc(Panc, NSTATE,ncat);
    free_piset(piset,maxQ);
    free_piset(rootset,ncat);
    free_piset(rootp,nmajor);
    free(rho);
    free(rhop);
    free(rmult);
    free(infilename);
    free(outfilename);
    free(treefilename);
    free(pfilename);
    free(params);
  }
  return EXIT_SUCCESS;
  
}/*main*/

