/*header for ML_genedata.c*/

/*  ML_genedata, a program for maximum likelihood modelling of gene family presence/absence data
 *  Copyright (C) 2007 by Matthew Spencer
 *
 *  This program is free software; you may redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  For any other enquiries write to Matthew Spencer, School of Biological Sciences,
 *  University of Liverpool, Crown Street, Liverpool, L69 7ZB, UK.
 *
 *  Or send E-mail to m.spencer@liverpool.ac.uk
 * 
 *---------------------------------------------------------------------------
 Parts of this program (functions to read phylip file) are based on fastDNAml by Gary J. Olsen, which is in turn based
 *  partly on dnaml by Joseph Felsenstein
 *
 *  Copyright notice from dnaml:
 *
 *     version 3.3. (c) Copyright 1986, 1990 by the University of Washington
 *     and Joseph Felsenstein.  Written by Joseph Felsenstein.  Permission is
 *     granted to copy and use this program provided no fee is charged for it
 *     and provided that this copyright notice is not removed.
 *
------------------------------------------------------------------------------
 *
 * Parts of this program (functions to read tree file and data structures for trees) are based on Seq-Gen by Andrew Rambaut and Nick Grassly * Copyright notice for Seq-Gen:
 *  
   Sequence Generator - seq-gen, version 1.3.2
   Copyright (c)1996-2004, Andrew Rambaut & Nick Grassly
   Department of Zoology, University of Oxford			
   All rights reserved.                          

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote 
        products derived from this software without specific prior written 
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


   Any feedback is very welcome.
   http://evolve.zoo.ox.ac.uk/software/Seq-Gen/
   email: andrew.rambaut@zoo.ox.ac.uk
*/

/*#define DEBUG*//*for testinput.txt, test.tree, should give -l=13.927451 before optimizing*/
/*#define DEBUG2*//*for testinput2.txt, test2.tree, should give -l=1.465987 before optimizing*/
/*#define DEBUG3*//*for test_ML_fourtaxa_binary.phy, test_fourtaxa_c.tree, should give -l=77.401382 before optimizing*/
/*#define BRIAN_DEBUG*//*for Bacillus.phy, Bacillus.tree, should give -l=41216.450222 before optimizing*/
/*#define BRIAN_DEBUG2*//*for Bacillus.phy, Bacillus_case2.tree, should give -l=37878.403742 before optimizing*/
/*#define BRIAN_DEBUG3*//*for Bacillus.phy, Bacillus_case2.tree, as BRIAN_DEBUG2 but with scaled Q matrices. With -f -c1, should get log likelihood -41497.228040. Should be the same as -f -oN -c1 with no debug flag*/

#define PROGRAM_NAME "ML_genedata"
#define PROGRAM_VERSION "0.4"
#define COPYRIGHT "Copyright Matthew Spencer, University of Liverpool, 2008 (m.spencer@liverpool.ac.uk)"
#define NSTATE 2 /*number of states: 0 and 1 for gene presence/absence data*/
#define TRUE             1
#define FALSE            0
#define LEN		1000 /*length of file name strings*/
#define nmlngth         60   /*max number of characters in species name */
#define MAXEDGE 100. /*maximum edge length allowed*/
#define MINEDGE 1e-6 /*minimum edgel length allowed*/
#define OPTTOL 1e-6 /*convergence criterion for log likelihood*/
#define MAXOPTIT 500 /*max number of optimization iterations (each iteration is an edge sweep and a multidimensional optimization of other parameters)*/
#define MAXMULTIT 100/*max iterations of multidimensional optimizer per iteration*/

#define MIN(a,b) ((a)>(b)?(b):(a))
#define MAX(a,b) ((a)>(b)?(a):(b))

enum {SIMPLEX,QNEWT};
enum {WORKING,CONVERGED,EXCEEDED};

typedef int boolean;

/*structure to hold genome data*/
typedef struct
{
  int **gdata;
  int ntaxa;
  int ngenes;
  int nunique;/*number of unique genes*/
  int nhidden;/*number of unobservable patterns*/
  char **names;
  int *weights;/*weight of this unique pattern*/
  int *match;/*here, we use these to store the index of an identical pattern*/
} genomedata;


/*structure to hold settings*/
typedef struct
{
  boolean printHelp;/*show help and exit?*/
  boolean quiet;/*noisy output?*/
  boolean printlikes;/*print gene family likelihoods?*/
  boolean edgeopt;/*optimize edge lengths?*/
  boolean parmopt;/*optimize other parameters?*/
  boolean printdata;/*print the input data and the unobservable patterns?*/
  boolean startparms;/*specify starting values for parameters?*/
  boolean simulate;/*generate simulated data?*/
  int sgf;/*number of simulated gene families*/
  int ctype;/*what kind of conditioning on observability?*/
  int optchoice;/*what optimization method for parameters other than edge lengths?*/
  int gammacat;/*number of gamma categories*/
  double edgescale;/*constant to multiply edge lengths by*/
} parameters;

void stationaryprob22(double **piset, double ***Qset, int i);
void Q_scale(double ***Qset, double **piset, int i, int nstate);
double get_treelike(TTree *tree, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double **rootset, genomedata *gd, double *rho, double *rmult, double *edgescale, int preorder);
double vertexlike_subtrees(TTree *tree, TNode *node, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double *rho, genomedata *gd, double **piset, int LRind, double *rmult, double *edgescale);/*calculate log likelihood at node using upper and lower subtrees*/
void print_Qset(double ***Qset, int nQ, int nstate, FILE *out_fv);
void write_parms(FILE *out_fv,double ***Qset,double *rhop,double **rootp,int nmajor,int gammacat,int maxQ,double alpha);
void rhofromrhop(double *rho, double *rhop, int gammacat, int nmajor);
void rootfromrootp(double **rootset, double **rootp, int gammacat,int nmajor, int nstate);
void print_rho(double *rho, double *rhop, int gammacat, int nmajor);
void print_root(double **rootset, double **rootp, int gammacat, int nmajor,int nstate);
void set_rmult(double *rmult, double alpha, int gammacat, int nmajor);
