/*this file contains optimization routines for parameter estimation*/

/*  ML_genedata, a program for maximum likelihood modelling of gene family presence/absence data
 *  Copyright (C) 2007 by Matthew Spencer
 *
 *  This program is free software; you may redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  For any other enquiries write to Matthew Spencer, School of Biological Sciences,
 *  University of Liverpool, Crown Street, Liverpool, L69 7ZB, UK.
 *
 *  Or send E-mail to m.spencer@liverpool.ac.uk
 * 
 *---------------------------------------------------------------------------
 Parts of this program (functions to read phylip file) are based on fastDNAml by Gary J. Olsen, which is in turn based
 *  partly on dnaml by Joseph Felsenstein
 *
 *  Copyright notice from dnaml:
 *
 *     version 3.3. (c) Copyright 1986, 1990 by the University of Washington
 *     and Joseph Felsenstein.  Written by Joseph Felsenstein.  Permission is
 *     granted to copy and use this program provided no fee is charged for it
 *     and provided that this copyright notice is not removed.
 *
------------------------------------------------------------------------------
 *
 * Parts of this program (functions to read tree file and data structures for trees) are based on Seq-Gen by Andrew Rambaut and Nick Grassly * Copyright notice for Seq-Gen:
 *  
   Sequence Generator - seq-gen, version 1.3.2
   Copyright (c)1996-2004, Andrew Rambaut & Nick Grassly
   Department of Zoology, University of Oxford			
   All rights reserved.                          

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote 
        products derived from this software without specific prior written 
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


   Any feedback is very welcome.
   http://evolve.zoo.ox.ac.uk/software/Seq-Gen/
   email: andrew.rambaut@zoo.ox.ac.uk
*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <float.h>
#include <gsl/gsl_multimin.h>
#include "treefile.h"
#include "opt_funcs.h"

typedef struct{/*structure to pass parameters we need for optimization to GSL functions*/
  TTree *tree;
  int maxQ,ncat,nmajor,gammacat,ngenes,nopt,nstates;
  double ***Qset,**Pleft,**Pright,***Panc,**rootp,**rootset,**piset,*rhop,*rho,*rmult,*edgescale,*alpha;
  genomedata *gd;
  gsl_vector *xv;
}optparms;

int edge_brk(TTree *tree, TNode *node, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double *rho, double *rmult, double *edgescale,genomedata *gd, double **rootset, int LRind, double *ep, double *a, double *b, double *c, double *la, double *lb, double *lc);/*bracket the minimum in negative log likelihood for edge length*/
double edge_inc(double x,double step,int dir);
int edge_lim(double x, int dir);
double tree_f(const gsl_vector *v, void *params);
void tree_df(const gsl_vector *v, void *params, gsl_vector *df);
void tree_fdf (const gsl_vector *x, void *params, double *f, gsl_vector *df);
void set_optparms(double ***Qset,double **piset,double *rhop,double *rho,double *rmult,double **rootp,double **rootset,double *alpha,const gsl_vector *v,int maxQ,int nmajor, int gammacat);
void print_gradient(gsl_vector *theta, gsl_vector *v, gsl_vector *df, optparms *p, FILE *out_fv, double **Jac);
void print_Hessian(optparms *p, double **Jac, double **Hess, FILE *out_fv);
void fill_optparms(optparms *treeparms, TTree *tree, int ncat, int nmajor, int gammacat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double **rootp, double **rootset, genomedata *gd, double *rhop, double *rho, double *rmult, double *edgescale, double *alpha, double **piset, int maxQ);
void transform_parameters(gsl_vector *x, int maxQ, double ***Qset, double *rhop, int nmajor, double **rootp, int gammacat, double *alpha);
void evaluate_gradient(optparms *p, gsl_vector *v, gsl_vector *df);
void evaluate_Hessian(optparms *p, gsl_vector *v, double **Hess, double treeloglike);
void get_Jacobian(optparms *p,gsl_vector *v, gsl_vector *theta, double **Jac);
void v3tree_df(const gsl_vector *v, void *params, gsl_vector *df);

/*alter an edge x down to x-step if dir is negative, up to x+step otherwise. Don't go outside bounds [MINEDGE,MAXEDGE]*/
double edge_inc(double x,double step,int dir)
{
  if(dir<0)
    /*    return(MAX(x-step,0.));*/
    return(MAX(x-step,MINEDGE));/*MS 7/8/08: we sometimes saw numerical problems with edges of zero*/
  else 
    return(MIN(x+step,MAXEDGE));
}/*edge_inc*/

/*did we hit limit of edge length range for edge x moving in direction dir negative for down, otherwise up?*/
int edge_lim(double x, int dir)
{
  if(dir<0)
    /*  return(x<=0.);*/
    return(x<=MINEDGE);/*MS 7/8/08: we sometimes saw numerical problems with edges of zero*/
  else return(x>=MAXEDGE);
}/*edge_lim*/

/*bracket the minimum in negative log likelihood for a single edge*/
/*ep is pointer to edge length we're optimizing, with current value b and current negative log likelihood lb*/
/*call this from edge_opt and it sets a and c, the min and max of the interval, and la and lc (corresponding negative log likelihoods. Will also move b and lb. Returns 0 if we bracketed, 1 if not. Also sets edge length *ep to the value giving the smallest negative log likelihood out of the triple [a,b,c]*/
int edge_brk(TTree *tree, TNode *node,int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double *rho, double *rmult, double *edgescale, genomedata *gd, double **rootset, int LRind, double *ep, double *a, double *b, double *c, double *la, double *lb, double *lc)
{

#define STEPINIT 0.01 /*initial step size for minimum bracketing*/
#define ALPHA 1.1 /*step size multiplier (increase power each time)*/

  double step,alpha;
  int bk,dir,bkres;
  double *bp,*lp;

  /*move out from current point*/

  step=STEPINIT;
  alpha=ALPHA;

  bkres=0;
  *a=edge_inc(*b,step,-1);
  *c=edge_inc(*b,step,1);

  *ep=*a;/*set to lower value*/
  *la=-vertexlike_subtrees(tree,node,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,gd,rootset,LRind, rmult,edgescale);/*get log likelihood*/
  /*  printf("Lower value %f, -l=%f\n",*ep,*la);*/
  *ep=*c;/*set to upper value*/
  *lc=-vertexlike_subtrees(tree,node,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,gd,rootset,LRind,rmult,edgescale);/*get log likelihood*/
  /*  printf("Upper value %f, -l=%f\n",*ep,*lc);*/
  /*printf("\n[a,b,c]=[%f,%f,%f]\n",*a,*b,*c);
    printf("[la,lb,lc]=[%f,%f,%f]\n",*la,*lb,*lc);*/



  if((*lb<*la) && (*lb<*lc)){/*already bracketing*/
    /*printf("already bracketing\n\n");*/
    *ep=*b;/*set edge to min value*/
    return(bkres);
  }
  else{bk=0;
    /*find the downhill direction*/
    if(*la<*lc)
    {/*set c=b,b=a,decrease a to bracket*/
      *c=*b;
      *b=*a;
      *lc=*lb;
      *lb=*la;
      bp=a;/*points to edge length*/
      lp=la;/*points to corresponding negative log likelihood*/
      dir=-1;
      /*      printf("will decrease a\n");*/
    }
    else{/*set a=b,b=c,increase c*/
      *a=*b;
      *b=*c;
      *la=*lb;
      *lb=*lc;
      bp=c;
      lp=lc;
      dir=1;
      /*      printf("will increase c\n");*/
    }

    if(edge_lim(*bp,dir)){/*check: are we already at the limit in the downhill direction?*/
      /*      printf("hit limit\n\n");*/
      *ep=*bp;/*set to the limit*/
      bk=1;
      bkres=1;
    }

    while(!bk){/*compute new point*/
      *bp=edge_inc(*bp,alpha*step,dir);
      *ep=*bp;
      *lp=-vertexlike_subtrees(tree,node,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,gd,rootset,LRind,rmult,edgescale);/*get log likelihood*/
      /* printf("[a,b,c]=[%f,%f,%f]\n",*a,*b,*c);
	 printf("[la,lb,lc]=[%f,%f,%f]\n",*la,*lb,*lc);*/

      if((*lb<*la) && (*lb<*lc)){/*test: are we bracketing?*/
	/*	printf("bracketed\n\n");*/
	*ep=*b;/*set edge to min value*/
	bk=1;
      }
      else if(edge_lim(*bp,dir)){
	/*	printf("hit limit\n\n");*/
	*ep=*bp;/*set to limit*/
	bk=1;
	bkres=1;
      }
      else{/*if not, increase step size, change relevant edge and try again*/
	alpha=alpha*alpha;
	/*	printf("increasing step\n");*/
      }
    }
  }
  return(bkres);
}/*edge_brk*/

/*optimize an edge length in preorder*/
/*uses golden section (if this is too slow, maybe implement Brent's method later)*/
/*treeloglike has been evaluated at the current edge length, which uses the left edge length and its associated transition rates (node->Qind1,node->length1) if LRind is 1, and the right edge length and transition rates (node->Qind2,node->length2) if LRind is 2*/
void edge_opt(TTree *tree, TNode *node, double treeloglike, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double *rho, double *rmult, double *edgescale, genomedata *gd, double **rootset, int LRind)
{
#define C 0.381966011250105 /*golden ratio number: (3-sqrt(5))/2*/
  
  double tol=sqrt(DBL_EPSILON);/*tolerance criterion*/
  double a,b,c,d,la,lb,lc,ld,oldb,oldt;
  double *ep;
  
  if(LRind==1){
    b=node->length1;/*current edge length*/
    ep=&node->length1;/*pointer to the edge length we're optimizing*/
  }
  else{
    b=node->length2;
    ep=&node->length2;
  }
  lb=-treeloglike;
  oldt=b;/*remember the original edge length*/

  /*printf("\nOptimizing edge, initial length %f, -l=%f\n",b,lb);*/

  /*bracket the minimum*/
  a=b;
  c=b;
  la=lb;
  lc=lb;
  if(!edge_brk(tree,node,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,rmult,edgescale,gd,rootset,LRind,ep,&a,&b,&c,&la,&lb,&lc)){/*only optimize if we bracketed*/
    /*printf("Initial triple [%f,%f,%f]\n",a,b,c);
      printf("[la,lb,lc]=[%f,%f,%f]\n",la,lb,lc);*/
    d=b;
    while(fabs(a-c)>tol*(fabs(b)+fabs(d))){/*iterate until tolerance reached:*/
       if(fabs(b-a)>fabs(c-b)){/*choose the larger segment*/
	 d=b-C*(b-a);/*choose new point within segment*/
	 oldb=b;/*want order a,b,d,c*/
	 b=d;
	 d=oldb;
	 ld=lb;
	 *ep=b;/*evaluate negative log likelihood of new point*/
	 lb=-vertexlike_subtrees(tree,node,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,gd,rootset,LRind,rmult,edgescale);/*get log likelihood*/
	 
       }
       else{
	 d=b+C*(c-b);
	 *ep=d;/*evaluate negative log likelihood of new point*/
	 ld=-vertexlike_subtrees(tree,node,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,gd,rootset,LRind,rmult,edgescale);/*get log likelihood*/
       }

       /*printf("Current quad [%f,%f,%f,%f]\n",a,b,d,c);
	 printf("[la,lb,ld,lc]=[%f,%f,%f,%f]\n",la,lb,ld,lc);*/

       if(ld<lb){/*new triple b,d,c*/
	 a=b;
	 b=d;
	 la=lb;
	 lb=ld;
       }
       else{/*new triple a,b,d*/
	 c=d;
	 lc=ld;
       }

       /*       printf("Current triple [%f,%f,%f]\n",a,b,c);
		printf("[la,lb,lc]=[%f,%f,%f]\n",la,lb,lc);*/
    }
    *ep=b;/*when converged, set edge to best of current values, which will always be b*/
  }
  else/*we didn't bracket so no optimization*/
  {
    /*printf("Did not bracket: using edge length %f\n",*ep);*/
    b=*ep;
  }
  lb=-vertexlike_subtrees(tree,node,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,gd,rootset,LRind,rmult,edgescale);/*final call to vertexlike_subtrees to make sure the upper subtree likelihoods are correct*/
  /*printf("final edge length %f, -l=%f\n",b,lb);*/

  /*change edge back to initial value: debugging*/
  /* *ep=oldt;*/
  /*lb=vertexlike_subtrees(tree,node,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rho,gd,LRind);*//*final call to vertexlike_subtrees to make sure the upper subtree likelihoods are correct*/
  /*  printf("reverted edge length %f, -l=%f\n",*ep,lb);*/
}/*edge_opt*/

/*edge length optimization function. Didn't figure out how to get it to return the log likelihood at the very last vertex visited, which would save us one call to get_treelike.*/
void opt_edgeset(TTree *tree, TNode *node, int ncat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double **rootset, genomedata *gd, double *rho, double *rmult, double *edgescale)
{
  node->optflag=1;/*set flag of vertex whose descendant edges we want to optimize*/
  get_treelike(tree,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rootset,gd,rho,rmult,edgescale,1);/*call get_treelike to optimize these edges: last argument 1 means we do preorder up to the vertex of interest*/
  node->optflag=0;/*set flag back so we don't optimize again*/

  /*recurse in preorder*/
  if(node->branch1->tipNo==-1) opt_edgeset(tree,node->branch1,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rootset,gd,rho,rmult,edgescale);
  if(node->branch2->tipNo==-1) opt_edgeset(tree,node->branch2,ncat,ngenes,nstates,Qset,Pleft,Pright,Panc,rootset,gd,rho,rmult,edgescale);
}/*opt_edgeset*/

/*optimize parameters other than edge lengths using GSL*/
double tree_opt(TTree *tree,int ncat, int nmajor, int gammacat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double **rootp, double **rootset, genomedata *gd, double *rhop, double *rho, double *rmult, double *edgescale, double *alpha, double **piset,parameters *params,int maxQ)
{
  size_t iter = 0;
  int status;
  optparms treeparms;
  const gsl_multimin_fminimizer_type *T;/*for simplex*/
  gsl_multimin_fminimizer *s=NULL;
  const gsl_multimin_fdfminimizer_type *Tg;/*for quasi-Newton*/
  gsl_multimin_fdfminimizer *sg=NULL;
  gsl_vector *ss,*x;
  gsl_multimin_function tree_func;/*for simplex*/
  gsl_multimin_function_fdf tree_funcg;/*for quasi-Newton*/
  double size,treeloglike;

  fill_optparms(&treeparms, tree, ncat, nmajor, gammacat, ngenes, nstates, Qset, Pleft, Pright, Panc, rootp, rootset, gd, rhop, rho, rmult, edgescale, alpha, piset, maxQ);/*fill the optparms structure*/
  x = gsl_vector_alloc(treeparms.nopt);
  transform_parameters(x, maxQ, Qset, rhop, nmajor, rootp, gammacat, alpha);/*transform to scale on which we optimized*/
  if(params->optchoice==SIMPLEX){/*use simplex*/
    tree_func.n = treeparms.nopt;/*number of parameters to optimize*/
    tree_func.f = &tree_f;/*pointer to objective function*/
    tree_func.params=&treeparms;/*structure of parameters*/

    /*initial step sizes*/
    ss=gsl_vector_alloc(treeparms.nopt);
    gsl_vector_set_all(ss,1.);

    /*allocate optimizer*/
    T = gsl_multimin_fminimizer_nmsimplex;
    s = gsl_multimin_fminimizer_alloc (T,treeparms.nopt);
    /*printf ("Using GSL %s minimizer\n",gsl_multimin_fminimizer_name (s));     */

    gsl_multimin_fminimizer_set (s, &tree_func, x, ss);/*initialize optimizer*/
    do{/*iterate optimizer*/
      iter++;
      status = gsl_multimin_fminimizer_iterate (s);
      treeloglike=s->fval;
      if (status)
	break;
      size=gsl_multimin_fminimizer_size(s);
      status = gsl_multimin_test_size(size, 1e-2);/*convergence: size of simplex*/
      /*if (status == GSL_SUCCESS)
	printf ("Minimum found at:\n");
	printf ("%5d %.5f %.5f %10.5f, size=%.3f\n", iter,exp(gsl_vector_get (s->x, 0)),exp(gsl_vector_get (s->x, 1)),s->fval,size);*/
      }
    while (status == GSL_CONTINUE && iter < MAXMULTIT);
    set_optparms(Qset,piset,rhop,rho,rmult,rootp,rootset,alpha,s->x,maxQ,nmajor,gammacat);/*set the optimized values, rescale Q matrices, set stationary probabilities, etc*/
    gsl_multimin_fminimizer_free (s);
    gsl_vector_free(ss);
  }
  else{/*use quasi-Newton*/
    treeparms.xv=gsl_vector_alloc(treeparms.nopt);/*need a vector we can modify for numerical derivatives*/

    tree_funcg.n = treeparms.nopt;/*number of parameters to optimize*/
    tree_funcg.f = &tree_f;/*pointer to objective function*/
    tree_funcg.df=&tree_df;/*pointer to gradient function*/
    tree_funcg.fdf=&tree_fdf;/*pointer to objective and gradient function*/
    tree_funcg.params=&treeparms;/*structure of parameters*/
    Tg = gsl_multimin_fdfminimizer_vector_bfgs;
    sg = gsl_multimin_fdfminimizer_alloc (Tg, treeparms.nopt);
    /*printf ("Using GSL %s minimizer\n",gsl_multimin_fdfminimizer_name (sg));     */
    gsl_multimin_fdfminimizer_set (sg, &tree_funcg, x, 1., 0.1);/*initial step size 1 was guess. tol 0.1 for line search recommended by GSL doc*/
    do{/*iterate optimizer*/
      iter++;
      status = gsl_multimin_fdfminimizer_iterate(sg);
      treeloglike=sg->f;
      if (status)
	break;
      status = gsl_multimin_test_gradient (sg->gradient, 1e-3);/*convergence? not worth a very stringent test because we iterate this several times if we're doing edges as well*/
      }
    while (status == GSL_CONTINUE && iter < MAXMULTIT);
    set_optparms(Qset,piset,rhop,rho,rmult,rootp,rootset,alpha,sg->x,maxQ,nmajor,gammacat);/*set the optimized values, rescale Q matrices, set stationary probabilities, etc*/
    gsl_vector_free(treeparms.xv);
    gsl_multimin_fdfminimizer_free (sg);
  }
  gsl_vector_free (x);
  return -treeloglike;
}/*tree_opt*/

/*evaluate negative log likelihood: for optimizing parameters other than edge lengths. Parameters other than the ones being optimized are passed in a pointer to a structure of type optparms.*/
double tree_f(const gsl_vector *v, void *params)
{
  optparms p;
  double treeloglike;
  
  p=*(optparms *)params;/*cast params to the type it really has*/
  set_optparms(p.Qset,p.piset,p.rhop,p.rho,p.rmult,p.rootp,p.rootset,p.alpha,v,p.maxQ,p.nmajor,p.gammacat);/*set parameters to current values*/
  treeloglike=get_treelike(p.tree,p.ncat,p.ngenes,p.nstates,p.Qset,p.Pleft,p.Pright,p.Panc,p.rootset,p.gd,p.rho,p.rmult,p.edgescale,0);/*get log likelihood, don't do preorder*/
  return -treeloglike;
}/*tree_f*/

/*gradient for parameters other than edge lengths. Parameters other than the ones being optimized are passed in a pointer params to a structure of type optparms. Gradient is written to df. Currently all done numerically.*/
void tree_df(const gsl_vector *v, void *params, gsl_vector *df)
{
  optparms p;

  p=*(optparms *)params;/*cast params to the type it really has*/
  gsl_vector_memcpy(p.xv,v);/*copy parameter vector to a non-const version*/
  evaluate_gradient(&p,p.xv,df);/*evaluate gradient*/
}/*tree_df*/

/* Compute both function f and gradient df for tree log likelihood. Params is a pointer to a structure of type optparms, containing the parameters not being optimized.*/
void tree_fdf (const gsl_vector *x, void *params, double *f, gsl_vector *df) 
{
  *f = tree_f(x, params); 
  tree_df(x, params, df);
}/*tree_fdf*/


/*translate vector of optimized parameters into the model rate matrices etc*/
void set_optparms(double ***Qset,double **piset,double *rhop,double *rho,double *rmult,double **rootp,double **rootset,double *alpha,const gsl_vector *v,int maxQ,int nmajor, int gammacat)
{
  int i;
  double rhosum=1.;

  for(i=0;i<maxQ;i++){/*rate matrices: back-transform*/
    Qset[i][1][0]=exp(gsl_vector_get(v,i));
    Qset[i][0][0]=-1.;
    Qset[i][0][1]=1.;/*assume gene loss rate was expressed relative to gain rate*/
    Qset[i][1][1]=-Qset[i][1][0];
    stationaryprob22(piset,Qset,i);/*stationary probabilities*/
    Q_scale(Qset,piset,i,NSTATE);/*scale Q matrix to one expected event per unit time at equilibrium*/
  }

  /*back-transform mixing probs*/
  for(i=0;i<nmajor-1;i++) rhosum+=exp(gsl_vector_get(v,i+maxQ));
  for(i=0;i<nmajor-1;i++) rhop[i]=exp(gsl_vector_get(v,i+maxQ))/rhosum;
  rhop[nmajor-1]=1./rhosum;
  for(i=0;i<nmajor;i++){/*back-transform root probabilities of absence*/
    rootp[i][0]=exp(gsl_vector_get(v,i+maxQ+nmajor-1))/(1+exp(gsl_vector_get(v,i+maxQ+nmajor-1)));
    rootp[i][1]=1.-rootp[i][0];
  }

  if(gammacat>1) *alpha=exp(gsl_vector_get(v,maxQ+2*nmajor-1));/*back-transform alpha*/
  else *alpha=1.;/*no gamma rates so this is arbitrary*/

  rhofromrhop(rho,rhop,gammacat,nmajor);/*cat prob=major cat prob * rate class prob*/
  /*print_rho(rho,rhop,gammacat,nmajor);*/
  rootfromrootp(rootset,rootp,gammacat,nmajor,NSTATE);/*root probabiility for each major cat * rate class*/

  set_rmult(rmult,*alpha,gammacat,nmajor);/*rate multipliers for categories*/
  /*print_root(rootset,rootp,gammacat,nmajor,NSTATE);*/
  /*write_parms(stdout,Qset,rhop,rootp,nmajor,gammacat,maxQ,*alpha);*/
}/*set_optparms*/

     
/*print gradient and Hessian for parameters other than edge lengths*/
void gradHessprint(TTree *tree,int ncat, int nmajor, int gammacat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double **rootp, double **rootset, genomedata *gd, double *rhop, double *rho, double *rmult, double *edgescale, double *alpha, double **piset,parameters *params,int maxQ, FILE *out_fv, double treeloglike)
{
  optparms treeparms;
  gsl_vector *x,*df,*theta,*v;
  int i;
  double **Jac, **Hess;

  fill_optparms(&treeparms, tree, ncat, nmajor, gammacat, ngenes, nstates, Qset, Pleft, Pright, Panc, rootp, rootset, gd, rhop, rho, rmult, edgescale, alpha, piset, maxQ);/*fill the optparms structure*/
  x=gsl_vector_alloc(treeparms.nopt);
  df=gsl_vector_alloc(treeparms.nopt);
  theta=gsl_vector_alloc(treeparms.nopt);
  v=gsl_vector_alloc(treeparms.nopt);
  treeparms.xv=gsl_vector_alloc(treeparms.nopt);/*need a vector we can modify for numerical derivatives*/

  /*allocate memory for Jacobian*/
  Jac=(double**)calloc(treeparms.nopt,sizeof(double*));
  if(Jac==NULL){
    (void)fprintf(stderr,"ERROR: calloc failed: gradHessprint");
    (void)exit(EXIT_FAILURE);
  }
  for(i=0;i<treeparms.nopt;i++){
    Jac[i]=(double*)calloc(treeparms.nopt,sizeof(double));
    if(Jac[i]==NULL){
      (void)fprintf(stderr,"ERROR: calloc failed: gradHessprint");
      (void)exit(EXIT_FAILURE);
    }
  }

  /*allocate memory for Hessian*/
  Hess=(double**)calloc(treeparms.nopt,sizeof(double*));
  if(Hess==NULL){
    (void)fprintf(stderr,"ERROR: calloc failed: gradHessprint");
    (void)exit(EXIT_FAILURE);
  }
  for(i=0;i<treeparms.nopt;i++){
    Hess[i]=(double*)calloc(treeparms.nopt,sizeof(double));
    if(Hess[i]==NULL){
      (void)fprintf(stderr,"ERROR: calloc failed: gradHessprint");
      (void)exit(EXIT_FAILURE);
    }
  }

  transform_parameters(v, maxQ, Qset, rhop, nmajor, rootp, gammacat, alpha);/*transform to scale on which we optimized*/
  gsl_vector_memcpy(treeparms.xv,v);/*copy parameter vector*/
  evaluate_gradient(&treeparms,v,df);/*evaluate the gradient*/
  for(i=0;i<treeparms.nopt;i++) gsl_vector_set(df,i,-gsl_vector_get(df,i));/*we need to change the sign of the gradient*/
  get_Jacobian(&treeparms,v,theta,Jac);/*get Jacobian and back-transformed parameters*/
  print_gradient(theta,v,df,&treeparms,out_fv,Jac);/*print out gradient*/
  evaluate_Hessian(&treeparms,v,Hess,treeloglike);/*evaluate the Hessian*/
  print_Hessian(&treeparms,Jac,Hess,out_fv);/*print out Hessian*/

  /*free memory*/
  for(i=0;i<treeparms.nopt;i++){
    free(Hess[i]);
    free(Jac[i]);
  }
  free(Jac);
  free(Hess);
  gsl_vector_free(treeparms.xv);
  gsl_vector_free(x);
  gsl_vector_free(df);
  gsl_vector_free(theta);
  gsl_vector_free(v);
}/*gradHessprint*/


/*transform parameters for optimization*/
void transform_parameters(gsl_vector *x, int maxQ, double ***Qset, double *rhop, int nmajor, double **rootp, int gammacat, double *alpha)
{
  int i;

  for(i=0;i<maxQ;i++) gsl_vector_set (x,i,log(Qset[i][1][0]/Qset[i][0][1]));/*relative q_{10} in category i, log transformed*/
  for(i=0;i<nmajor-1;i++) gsl_vector_set(x,i+maxQ,log(rhop[i]/rhop[nmajor-1]));/*mixing probs transform to ncat-1 independent values on real line*/
  for(i=0;i<nmajor;i++) gsl_vector_set(x,i+maxQ+nmajor-1,log(rootp[i][0]/(1-rootp[i][0])));/*root probability of absence in category i, logistic transform*/
   if(gammacat>1) gsl_vector_set(x,maxQ+2*nmajor-1,log(*alpha));/*shape parameter, log transform*/

}/*transform_parameters*/

/*fill the structure optparms with data needed for optimization*/
void fill_optparms(optparms *treeparms, TTree *tree, int ncat, int nmajor, int gammacat, int ngenes, int nstates, double ***Qset, double **Pleft, double **Pright, double ***Panc, double **rootp, double **rootset, genomedata *gd, double *rhop, double *rho, double *rmult, double *edgescale, double *alpha, double **piset, int maxQ)
{
  treeparms->tree=tree;
  treeparms->ncat=ncat;
  treeparms->nmajor=nmajor;
  treeparms->gammacat=gammacat;
  treeparms->ngenes=ngenes;
  treeparms->nstates=nstates;
  treeparms->Qset=Qset;
  treeparms->Pleft=Pleft;
  treeparms->Pright=Pright;
  treeparms->Panc=Panc;
  treeparms->rhop=rhop;
  treeparms->rho=rho;
  treeparms->rmult=rmult;
  treeparms->edgescale=edgescale;
  treeparms->gd=gd;
  treeparms->rootp=rootp;
  treeparms->rootset=rootset;
  treeparms->alpha=alpha;
  treeparms->piset=piset;
  treeparms->maxQ=maxQ;
  if(gammacat==1)treeparms->nopt=maxQ+2*nmajor-1;/*no gamma rates*/
  else treeparms->nopt=maxQ+2*nmajor;/*gamma rates, so optimize alpha too*/
}/*fill_optparms*/ 

void evaluate_gradient(optparms*p, gsl_vector *v, gsl_vector *df)
{
  double h=pow(DBL_EPSILON,0.333333);/*step size: this should really be inside the loop, but in a test case that works worse over many edge-length/parameter iterations, so left it here*/
  volatile double temp;
  double x,fplus,fminus;
  int i;

  for(i=0;i<p->nopt;i++){/*each parameter in turn*/
    x=gsl_vector_get(v,i);

    /*set the step size*/
    temp=x+h;/*we want to ensure h is exactly representable*/
    h=temp-x;

    /*evaluate the gradient and write to df*/
    gsl_vector_set(p->xv,i,x+h);
    set_optparms(p->Qset,p->piset,p->rhop,p->rho,p->rmult,p->rootp,p->rootset,p->alpha,p->xv,p->maxQ,p->nmajor,p->gammacat);/*set parameters to current values*/
    fplus=-get_treelike(p->tree,p->ncat,p->ngenes,p->nstates,p->Qset,p->Pleft,p->Pright,p->Panc,p->rootset,p->gd,p->rho,p->rmult,p->edgescale,0);/*get log likelihood, don't do preorder*/
    gsl_vector_set(p->xv,i,x-h);
    set_optparms(p->Qset,p->piset,p->rhop,p->rho,p->rmult,p->rootp,p->rootset,p->alpha,p->xv,p->maxQ,p->nmajor,p->gammacat);/*set parameters to current values*/
    fminus=-get_treelike(p->tree,p->ncat,p->ngenes,p->nstates,p->Qset,p->Pleft,p->Pright,p->Panc,p->rootset,p->gd,p->rho,p->rmult,p->edgescale,0);/*get log likelihood, don't do preorder*/
    gsl_vector_set(df,i,(fplus-fminus)/(2.*h));/*central difference formula*/
    gsl_vector_set(p->xv,i,x);/*set back to original value*/
  }
}/*evaluate_gradient*/

/*Hessian with respect to transformed parameters*/
void evaluate_Hessian(optparms *p, gsl_vector *v, double **Hess, double treeloglike)
{
  int i,j;
  double hi,hj,x,y,fpp,fpm,fmp,fmm;
  volatile double temp;

  for(i=0;i<p->nopt;i++){/*rows of Hessian*/
    x=gsl_vector_get(v,i);
    hi=pow(DBL_EPSILON,0.333333);/*set the step size*/
    temp=x+hi;/*we want to ensure h is exactly representable*/
    hi=temp-x;

    /*get diagonal element of Hessian*/
    gsl_vector_set(p->xv,i,x+hi);
    set_optparms(p->Qset,p->piset,p->rhop,p->rho,p->rmult,p->rootp,p->rootset,p->alpha,p->xv,p->maxQ,p->nmajor,p->gammacat);/*set parameters to current values*/
    fpp=get_treelike(p->tree,p->ncat,p->ngenes,p->nstates,p->Qset,p->Pleft,p->Pright,p->Panc,p->rootset,p->gd,p->rho,p->rmult,p->edgescale,0);/*get log likelihood, don't do preorder*/
    gsl_vector_set(p->xv,i,x-hi);
    set_optparms(p->Qset,p->piset,p->rhop,p->rho,p->rmult,p->rootp,p->rootset,p->alpha,p->xv,p->maxQ,p->nmajor,p->gammacat);/*set parameters to current values*/
    fmm=get_treelike(p->tree,p->ncat,p->ngenes,p->nstates,p->Qset,p->Pleft,p->Pright,p->Panc,p->rootset,p->gd,p->rho,p->rmult,p->edgescale,0);/*get log likelihood, don't do preorder*/
    Hess[i][i]=(fpp-2.*treeloglike+fmm)/(hi*hi);/*central difference formula*/
    gsl_vector_set(p->xv,i,x);/*set back to original value*/

    for(j=i+1;j<p->nopt;j++){/*off-diagonal columns of Hessian*/
      y=gsl_vector_get(v,j);
      hj=pow(DBL_EPSILON,0.333333);/*set the step size*/
      temp=y+hj;/*we want to ensure h is exactly representable*/
      hj=temp-y;

      /*++ pattern of parameter changes*/
      gsl_vector_set(p->xv,i,x+hi);
      gsl_vector_set(p->xv,j,y+hj);
      set_optparms(p->Qset,p->piset,p->rhop,p->rho,p->rmult,p->rootp,p->rootset,p->alpha,p->xv,p->maxQ,p->nmajor,p->gammacat);/*set parameters to current values*/
      fpp=get_treelike(p->tree,p->ncat,p->ngenes,p->nstates,p->Qset,p->Pleft,p->Pright,p->Panc,p->rootset,p->gd,p->rho,p->rmult,p->edgescale,0);/*get log likelihood, don't do preorder*/

      /*+- pattern*/
      gsl_vector_set(p->xv,i,x+hi);
      gsl_vector_set(p->xv,j,y-hj);
      set_optparms(p->Qset,p->piset,p->rhop,p->rho,p->rmult,p->rootp,p->rootset,p->alpha,p->xv,p->maxQ,p->nmajor,p->gammacat);/*set parameters to current values*/
      fpm=get_treelike(p->tree,p->ncat,p->ngenes,p->nstates,p->Qset,p->Pleft,p->Pright,p->Panc,p->rootset,p->gd,p->rho,p->rmult,p->edgescale,0);/*get log likelihood, don't do preorder*/

      /*-+ pattern*/
      gsl_vector_set(p->xv,i,x-hi);
      gsl_vector_set(p->xv,j,y+hj);
      set_optparms(p->Qset,p->piset,p->rhop,p->rho,p->rmult,p->rootp,p->rootset,p->alpha,p->xv,p->maxQ,p->nmajor,p->gammacat);/*set parameters to current values*/
      fmp=get_treelike(p->tree,p->ncat,p->ngenes,p->nstates,p->Qset,p->Pleft,p->Pright,p->Panc,p->rootset,p->gd,p->rho,p->rmult,p->edgescale,0);/*get log likelihood, don't do preorder*/

      /*-- pattern*/
      gsl_vector_set(p->xv,i,x-hi);
      gsl_vector_set(p->xv,j,y-hj);
      set_optparms(p->Qset,p->piset,p->rhop,p->rho,p->rmult,p->rootp,p->rootset,p->alpha,p->xv,p->maxQ,p->nmajor,p->gammacat);/*set parameters to current values*/
      fmm=get_treelike(p->tree,p->ncat,p->ngenes,p->nstates,p->Qset,p->Pleft,p->Pright,p->Panc,p->rootset,p->gd,p->rho,p->rmult,p->edgescale,0);/*get log likelihood, don't do preorder*/

      Hess[i][j]=(fpp-fpm-fmp+fmm)/(4.*hi*hj);/*central difference formula*/
      Hess[j][i]=Hess[i][j];/*Hessian is symmetrical*/

      gsl_vector_set(p->xv,i,x);/*set back to original value*/
      gsl_vector_set(p->xv,j,y);/*set back to original value*/
    }
  }
  /*test:print out Hessian*/
  /* printf("Hessian\n"); */
/*   for(i=0;i<p->nopt;i++){ */
/*       for(j=0;j<p->nopt;j++) printf("%.12f ",Hess[i][j]); */
/*       printf("\n"); */
/*   } */
}/*evaluate_Hessian*/

/*calculate gradient with respect to both original and transformed parameters and print it out*/
void print_gradient(gsl_vector *theta, gsl_vector *v, gsl_vector *df, optparms *p, FILE *out_fv, double **Jac)
{
  int i,j;
  double dl;

  fprintf(out_fv,"\nGradient for parameters other than edge lengths:\n");
  for(i=0;i<p->nopt;i++){/*each parameter in turn*/
    fprintf(out_fv,"Parameter %d, ",i);
    if(i<p->maxQ){/*Stationary probability of absence*/
      fprintf(out_fv,"stationary probability of absence for rate matrix %d, ",i);
    }
    else if(i<p->maxQ+p->nmajor-1){/*mixing prob*/
      fprintf(out_fv,"mixing probability for major category %d, ",i-p->maxQ);
    }
    else if(i<p->maxQ+p->nmajor-1+p->nmajor){/*root prob of absence*/
      fprintf(out_fv,"root probability of absence for major category %d, ",i-p->maxQ-p->nmajor+1);
    }
    else{/*shape parameter*/
      fprintf(out_fv,"shape parameter, ");
    }
    dl=0.;
    for(j=0;j<p->nopt;j++){/*multiply transpose of Jacobian by transformed gradient*/
      dl+=Jac[j][i]*gsl_vector_get(df,i);
    }
    fprintf(out_fv,"original value %.12f, transformed value %.12f, gradient %.12f\n",gsl_vector_get(theta,i),gsl_vector_get(v,i),dl);
  }
}/*print_gradient*/

/*back-transform and print out the Hessian*/
void print_Hessian(optparms *p, double **Jac, double **Hess, FILE *out_fv)
{
  int h,i,j,k;
  double y,z;

  fprintf(out_fv,"\nHessian for untransformed parameters, same order as gradient:\n");
  for(i=0;i<p->nopt;i++){
    for(j=0;j<p->nopt;j++){
      z=0.;
      for(h=0;h<p->nopt;h++){
	y=0.;
	for(k=0;k<p->nopt;k++) y+=Hess[h][k]*Jac[k][j];
	z+=y*Jac[h][i];
      }
      fprintf(out_fv,"%.12f ",z);
    }
    fprintf(out_fv,"\n");
  }
}/*print_Hessian*/

/*calculate Jacobian of transformed parameters with respect to original parameters, and vector of back-transformed parameters in the form for which we want the gradient and Hessian*/
void get_Jacobian(optparms *p,gsl_vector *v, gsl_vector *theta, double **Jac)
{
  int i,j;
  double x,xb,rhosum,muM;

  /*printf("Jacobian\n");*/
  rhosum=1.;
  for(i=0;i<p->nmajor-1;i++) rhosum+=exp(gsl_vector_get(v,i+p->maxQ));/*used in back-transforming mixing probs*/
  muM=1./rhosum;/*mixing prob for last category*/

  for(i=0;i<p->nopt;i++){/*each parameter in turn*/
      x=gsl_vector_get(v,i);
      if(i<p->maxQ){/*Stationary probability of absence*/
	xb=1.-1./(exp(x)+1.);/*back-transform parameter*/
	Jac[i][i]=1./(xb*(1.-xb));/*derivative of transformed parameter with respect to original*/

      }
      else if(i<p->maxQ+p->nmajor-1){/*mixing prob*/
	xb=exp(x)/rhosum;
	for(j=p->maxQ;j<p->maxQ+p->nmajor-1;j++){/*Jacobian isn't diagonal for mix probs*/
	  Jac[i][j]=1./muM;
	  if(i==j) Jac[i][j]+=1./xb;/*diagonal element*/
	}
      }
      else if(i<p->maxQ+p->nmajor-1+p->nmajor){/*root prob of absence*/
	xb=exp(x)/(1.+exp(x));
	Jac[i][i]=1./(xb*(1.-xb));
      }
      else{/*shape parameter*/
	xb=exp(x);
	Jac[i][i]=1./xb;
      }
      gsl_vector_set(theta,i,xb);/*store back-transformed parameter*/

      /*test: print out Jacobian*/
      /* for(j=0;j<p->nopt;j++){ */
/* 	printf("%.12f ",Jac[i][j]); */
/*       } */
/*       printf("\n"); */
    }
}/*get_Jacobian*/
