/*  
   Sequence Generator - seq-gen, version 1.3.2
   Copyright (c)1996-2004, Andrew Rambaut & Nick Grassly
   Department of Zoology, University of Oxford			
   All rights reserved.                          

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote 
        products derived from this software without specific prior written 
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


   Any feedback is very welcome.
   http://evolve.zoo.ox.ac.uk/software/Seq-Gen/
   email: andrew.rambaut@zoo.ox.ac.uk
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#include "global.h"
#include "treefile.h"

char treeErrorMsg[256];
int treeError;

TNode *avail=NULL;
long usedAvail=0;
long usedMalloc=0;

/* prototypes */

TNode *NewNode(TTree *tree, int ngenes, int nstates, int ncat, int storelike);
void InitTree(TTree *tree);
void CheckCapacity(TTree *tree, int required);
void DisposeNode(TNode *aNode);
void DisposeTreeNodes(TTree *tree);
void FreeNodes(TTree *tree, int nstates, int ncat, int gammacat);

char ReadToNextChar(FILE *fv);
void ReadUntil(FILE *fv, char stopChar, char *what);
TNode *ReadTip(FILE *fv, char ch, TTree *tree, int numNames, char **names, int ngenes, int nstates, int ncat, int storelike);
TNode *ReadNode(FILE *fv, TTree *tree, int numNames, char **names, int detectPolytomies, int ngenes, int nstates, int ncat, int storelike);
TNode *ReadBranch(FILE *fv, TTree *tree, int numNames, char **names, int ngenes, int nstates, int ncat, int storelike);
void WriteNode(FILE *fv, TTree *tree, TNode *node);
void print_CLike_node(TTree *tree, TNode *node, int ngenes, int nstates);

/* functions */

/*----------*/
TNode *NewNode(TTree *tree, int ngenes, int nstates, int ncat, int storelike)
{
	TNode *node;
	int i,j;

	if ( avail!=NULL ) {
		node=avail;
		avail=avail->next;
		
		usedAvail++;
	} else {
		if ( (node=malloc(sizeof(TNode)))==NULL ) {
			strcpy(treeErrorMsg, "Out of memory");
			return NULL;
		}
		usedMalloc++;
	}
	node->branch0=node->branch1=node->branch2=NULL;
	node->length0=node->length1=node->length2=0.0;
	node->param=0.0;
	node->tipNo=-1;
	node->optflag=0;
	/*allocate space for array of conditional likelihoods?*/
	if(storelike){
	  node->CLike=(double ***)calloc(ncat,sizeof(double**));
	  if(node->CLike==NULL){
	    (void)fprintf(stderr,"ERROR: calloc failed: node conditional likelihood");
	    (void)exit(EXIT_FAILURE);
	  }
	  for(i=0;i<ncat;i++){
	    node->CLike[i]=(double **)calloc(nstates,sizeof(double*));
	    if(node->CLike[i]==NULL){
	      (void)fprintf(stderr,"ERROR: calloc failed: node conditional likelihood");
	      (void)exit(EXIT_FAILURE);
	    }
	    for(j=0;j<nstates;j++){
	      node->CLike[i][j]=(double*)calloc(ngenes,sizeof(double));
	      if(node->CLike[i][j]==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed: node conditional likelihood");
		(void)exit(EXIT_FAILURE);
	      }
	    }
	  }
	  node->ULike=(double ***)calloc(ncat,sizeof(double**));
	  if(node->ULike==NULL){
	    (void)fprintf(stderr,"ERROR: calloc failed: node upper conditional likelihood");
	    (void)exit(EXIT_FAILURE);
	  }
	  for(i=0;i<ncat;i++){
	    node->ULike[i]=(double **)calloc(nstates,sizeof(double*));
	    if(node->ULike[i]==NULL){
	      (void)fprintf(stderr,"ERROR: calloc failed: node upper conditional likelihood");
	      (void)exit(EXIT_FAILURE);
	    }
	    for(j=0;j<nstates;j++){
	      node->ULike[i][j]=(double*)calloc(ngenes,sizeof(double));
	      if(node->ULike[i][j]==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed: node upper conditional likelihood");
		(void)exit(EXIT_FAILURE);
	      }
	    }
	  }
	  node->nULike=(double ***)calloc(ncat,sizeof(double**));
	  if(node->nULike==NULL){
	    (void)fprintf(stderr,"ERROR: calloc failed: node upper temp conditional likelihood");
	    (void)exit(EXIT_FAILURE);
	  }
	  for(i=0;i<ncat;i++){
	    node->nULike[i]=(double **)calloc(nstates,sizeof(double*));
	    if(node->nULike[i]==NULL){
	      (void)fprintf(stderr,"ERROR: calloc failed: node upper temp conditional likelihood");
	      (void)exit(EXIT_FAILURE);
	    }
	    for(j=0;j<nstates;j++){
	      node->nULike[i][j]=(double*)calloc(ngenes,sizeof(double));
	      if(node->nULike[i][j]==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed: node upper temp conditional likelihood");
		(void)exit(EXIT_FAILURE);
	      }
	    }
	  }
	}
	else{
	  node->CLike=NULL;
	  node->ULike=NULL;
	  node->nULike=NULL;
	}

	/*allocate space for indices of Q matrices*/
	if((node->Qind0=(int *)calloc(ncat,sizeof(int)))==NULL){
	  (void)fprintf(stderr,"ERROR: calloc failed: node Qind0\n");
	  (void)exit(EXIT_FAILURE);
	}
	if((node->Qind1=(int *)calloc(ncat,sizeof(int)))==NULL){
	  (void)fprintf(stderr,"ERROR: calloc failed: node Qind1\n");
	  (void)exit(EXIT_FAILURE);
	}
	if((node->Qind2=(int *)calloc(ncat,sizeof(int)))==NULL){
	  (void)fprintf(stderr,"ERROR: calloc failed: node Qind2\n");
	  (void)exit(EXIT_FAILURE);
	}
 
	node->next=tree->nodeList;	
	tree->nodeList=node;
	
	tree->numNodes++;

	return node;
} /* NewNode */


/*----------*/
void InitTree(TTree *tree)
{	
	tree->root=NULL;
	tree->nodeList=NULL;
	tree->numNodes=0;
	tree->numTips=0;
	tree->totalLength=0.0;
	tree->rooted=0;
	tree->lengths=-1;
} /* InitTree */


/*----------*/
TTree *NewTree()
{
	TTree *tree;
	
	if ( (tree=(TTree *)malloc(sizeof(TTree)))==NULL ) {
		strcpy(treeErrorMsg, "Out of memory creating tree.");
		return NULL;
	}
	tree->capacity=0;
	tree->names=NULL;
	tree->tips=NULL;
	tree->likes=NULL;
	CheckCapacity(tree, 1000);
	
	InitTree(tree);
	
	return tree;
} /* NewTree */


/*----------*/
void CheckCapacity(TTree *tree, int required)
{
	int i;
	int newCapacity = tree->capacity;
	char **newNames;
	TNode **newTips;
	
	while (newCapacity < required) {
		newCapacity += 1000;
	}
	
	newNames = (char**)CAllocMem(sizeof(char*)*newCapacity, "newNames", "CheckCapacity", 0);
	newTips = (TNode**)CAllocMem(sizeof(TNode*)*newCapacity, "newTips", "CheckCapacity", 0);
	
	for (i = 0; i < tree->capacity; i++) {
		newNames[i] = tree->names[i];
		newTips[i] = tree->tips[i];
	}
	
	for (i = tree->capacity; i < newCapacity; i++) {
		newNames[i] = NULL;
		newTips[i] = NULL;
	}
	
	if(tree->names!=NULL) free(tree->names);
	if(tree->tips!=NULL)  free(tree->tips);
	
	tree->names = newNames;
	tree->tips = newTips;
	tree->capacity = newCapacity;
} /* CheckCapacity */


/*----------*/
void DisposeNode(TNode *aNode)
{
	aNode->next=avail;
	avail=aNode;
} /* DisposeNode */
			

/*----------*/
void DisposeTreeNodes(TTree *tree)
{
	TNode *P, *O;
	
	if ( tree ) {
		P=tree->nodeList;
		while (P!=NULL) {
			O=P;
			P=P->next;
			DisposeNode(O);
		}
		tree->nodeList=NULL;
	}
} /* DisposeTreeNodes */


/*----------*/
void DisposeTree(TTree *tree)
{
	if (tree) {
		DisposeTreeNodes(tree);
		InitTree(tree);
	}
} /* DisposeTree */



/*MS version. 13/04/08*/
void FreeNodes(TTree *tree, int nstates, int ncat, int gammacat)
{
	TNode *P, *O;
	int i,j;

	if ( tree ) {
		P=tree->nodeList;
		while (P!=NULL) {
			O=P;
			P=P->next;
			if(O->CLike){
			  for(i=0;i<ncat*gammacat;i++){
			    for(j=0;j<nstates;j++) free(O->CLike[i][j]);
			    free(O->CLike[i]);
			  }
			  free(O->CLike);
			}
			if(O->ULike){
			  for(i=0;i<ncat*gammacat;i++){
			    for(j=0;j<nstates;j++) free(O->ULike[i][j]);
			    free(O->ULike[i]);
			  }
			  free(O->ULike);
			}
			if(O->nULike){
			  for(i=0;i<ncat*gammacat;i++){
			    for(j=0;j<nstates;j++) free(O->nULike[i][j]);
			    free(O->nULike[i]);
			  }
			  free(O->nULike);
			}
		      if(O->Qind0) free(O->Qind0);
		      if(O->Qind1) free(O->Qind1);
		      if(O->Qind2) free(O->Qind2);
		      free(O);
		      }

	}
	
} /*FreeNodes*/


/*----------*/
void FreeTree(TTree *tree, int nstates, int ncat, int gammacat)
{
  int i;
	if (tree) {
	  for(i=0;i<tree->numTips;i++) free(tree->names[i]); /*MS: added 11/04/08*/
	  free(tree->names); /*MS: added 10/04/08*/
	  free(tree->tips); /*MS: added 10/04/08*/
	  FreeNodes(tree,nstates,ncat,gammacat); /*MS: added 13/04/08*/

	  if(tree->likes){
	    for(i=0;i<(ncat*gammacat)+1;i++) free(tree->likes[i]); /*MS: added 30/4/08*/
	    free(tree->likes); /*MS: added 25/4/08*/
	  }
	  free(tree);
	}
} /* FreeTree */


/*----------*/
void WriteAvailInfo()
{
	TNode *P;
	int count;
	
	count=0;
	P=avail;
	while (P!=NULL) {
		P=P->next;
		count++;	
	}

	fprintf(stderr, "Avail: %d nodes - availed: %ld, malloced: %ld\n", count, usedAvail, usedMalloc);
} /* WriteAvailInfo */


int CountTrees(FILE *fv)
{
	int n;
		
	if (fv==NULL)
		return 0;
		
	n=0;
	while (!feof(fv)) {
		if (fgetc(fv)==';')
			n++;
	}
	rewind(fv);
	
	return n;
}

char ReadToNextChar(FILE *fv)
{
	char ch;
	
	ch=fgetc(fv);
	while (!feof(fv) && isspace(ch)) 
		ch=fgetc(fv);

	return ch;
}


void ReadUntil(FILE *fv, char stopChar, char *what)
{
	char ch;
	
	ch=fgetc(fv);
	while (!feof(fv) && ch!=stopChar && 
			ch!='(' && ch!=',' && ch!=':' && ch!=')' && ch!=';') 
		ch=fgetc(fv);

	if (feof(fv) || ch!=stopChar) {
		sprintf(treeErrorMsg, "%s missing", what);
		treeError=1;
	}
}


TNode *ReadTip(FILE *fv, char ch, TTree *tree, int numNames, char **names, int ngenes, int nstates, int ncat, int storelike)
{
	int i;
	char *P;
	char name[256];
	
	TNode *node;
	
	node=NewNode(tree,ngenes,nstates,ncat,storelike);
	i=0;
	
	P=name;
	while (!feof(fv) && ch!=':' && ch!=',' && ch!=')' && i<MAX_NAME_LEN) {
		if (!isspace(ch)) {
			*P=ch;
			i++;P++;
		}
		ch=fgetc(fv);
	}
	*P='\0';

	CheckCapacity(tree, tree->numTips);
	
	if (numNames == 0) {
		node->tipNo=tree->numTips;
		if (tree->names[node->tipNo]==NULL) {
			if ( (tree->names[node->tipNo]=(char *)malloc(MAX_NAME_LEN+1))==NULL ) {
				strcpy(treeErrorMsg, "Out of memory creating name.");
				return NULL;
			}
		}
		strcpy(tree->names[node->tipNo], name);
	} else {
	/* we already have some names so just look it up...*/
		i = 0;
		while (i < numNames && strcmp(name, names[i]) != 0)
			i++;

		if (i == numNames) {
			sprintf(treeErrorMsg, "Taxon names in trees for different partitions do not match.");
			return NULL;
		}
		
		node->tipNo=i;

		/*MS: INSERTED 13/4/08 otherwise WriteTree only works for tree 0*/
		if (tree->names[node->tipNo]==NULL) {
			if ( (tree->names[node->tipNo]=(char *)malloc(MAX_NAME_LEN+1))==NULL ) {
				strcpy(treeErrorMsg, "Out of memory creating name.");
				return NULL;
			}
		}
		strcpy(tree->names[node->tipNo], name);
	}
	/*--------------------*/
	
	tree->tips[node->tipNo]=node;
	tree->numTips++;

	while (!feof(fv) && ch!=':' && ch!=',' && ch!=')') 
		ch=fgetc(fv);

	if (feof(fv)) {
		sprintf(treeErrorMsg, "Unexpected end of file");
		return NULL;
	}
	ungetc(ch, fv);
	
	return node;
}


TNode *ReadNode(FILE *fv, TTree *tree, int numNames, char **names, int detectPolytomies, int ngenes, int nstates, int ncat, int storelike)
{
	TNode *node, *node2;
	char ch;
	
	if ((node=NewNode(tree, ngenes, nstates, ncat, storelike))==NULL)
		return NULL;

	if ((node2=ReadBranch(fv, tree, numNames, names, ngenes, nstates, ncat, storelike))==NULL)
		return NULL;
	node->branch1=node2;
	node2->branch0=node;
	node->length1=node2->length0;
	ReadUntil(fv, ',', "Comma");
	if (treeError)
		return NULL;
	
	if ((node2=ReadBranch(fv, tree, numNames, names, ngenes, nstates, ncat, storelike))==NULL)
		return NULL;
	node->branch2=node2;
	node2->branch0=node;
	node->length2=node2->length0;
	
	ch=fgetc(fv);
	while (!feof(fv) && ch!=':' && ch!=',' && ch!=')' && ch!=';') 
		ch=fgetc(fv);
		
	if (detectPolytomies && ch==',') {
		fprintf(stderr, "This tree contains nodes which aren't bifurcations. Resolve the node\n");
		fprintf(stderr, "with zero branch lengths to obtain correct results. This can be done\n");
		fprintf(stderr, "with a program called TreeEdit: http://evolve.zoo.ox.ac.uk/software/TreeEdit\n");
		exit(0);
	}

	if (feof(fv)) {
		sprintf(treeErrorMsg, "Unexpected end of file");
		return NULL;
	}
	ungetc(ch, fv);
	
	return node;
}


TNode *ReadBranch(FILE *fv, TTree *tree, int numNames, char **names, int ngenes, int nstates, int ncat, int storelike)
{
	char ch;
	double len, param=0.0;
	TNode *node;
	
	ch=ReadToNextChar(fv);
	if (ch=='(') {	/* is a node */
		node=ReadNode(fv, tree, numNames, names, 1, ngenes, nstates, ncat, storelike);
		ReadUntil(fv, ')', "Closing bracket");
		if (treeError)
			return NULL;
	} else {		/* is a tip */
		node=ReadTip(fv, ch, tree, numNames, names, ngenes, nstates, ncat, storelike);
	}
	
	ch=ReadToNextChar(fv);
	if (ch==':') {
		if (tree->lengths==0) {
			sprintf(treeErrorMsg, "Some branches don't have branch lengths");
			return NULL;
		} else 
			tree->lengths=1;
			
		if (fscanf(fv, "%lf", &len)!=1) {
			sprintf(treeErrorMsg, "Unable to read branch length");
			return NULL;
		}

		ch=ReadToNextChar(fv);
		if (ch=='[') {
			if (fscanf(fv, "%lf", &param)!=1) {
				sprintf(treeErrorMsg, "Unable to read branch parameter");
				return NULL;
			}
			ReadUntil(fv, ']', "Close square bracket");
		} else
			ungetc(ch, fv);
	} else {
		if (tree->lengths==1) {
			sprintf(treeErrorMsg, "Some branches don't have branch lengths");
			return NULL;
		} else 
			tree->lengths=0;
	
		len=0.0;
		ungetc(ch, fv);
	}
	node->length0=len;
	node->param=param;
	
	return node;
}	


/*if storelike is 1, allocate space for likelihoods. Otherwise not*/
void ReadTree(FILE *fv, TTree *tree, int treeNum, int numNames, char **names, int ngenes, int nstates, int ncat, int storelike)
{
	char ch;
	TNode *P;
	int i;

	treeError=0;
	tree->numNodes=0;
	tree->numTips=0;
	tree->rooted=1;
	tree->lengths=-1;
	
	ch=fgetc(fv);
	while (!feof(fv) && ch!='(' && ch!='[') 
		ch=fgetc(fv);

	if (ch == '[') {

		ch=fgetc(fv);
		while (!feof(fv) && ch!=',' && ch!='(') 
			ch=fgetc(fv);
			
		if (ch == ',') {
			ch=fgetc(fv);
			while (!feof(fv) && ch!='(') 
				ch=fgetc(fv);
		}
	}
	
	if (ch!='(' || (tree->root=ReadNode(fv, tree, numNames, names, 0, ngenes, nstates, ncat, storelike))==NULL) {
		fprintf(stderr, "Error reading tree number %d: %s.\n", treeNum, treeErrorMsg);
		exit(0);
	}
	
	ch=fgetc(fv);
	while (!feof(fv) && ch!=',' && ch!=')' && ch!=';') 
		ch=fgetc(fv);

	if (ch==',') {		/* the tree is unrooted */
		tree->rooted=0;
		if ((tree->root->branch0=ReadBranch(fv, tree, numNames, names, ngenes, nstates, ncat, storelike))==NULL) {
			fprintf(stderr, "Error reading tree number %d: %s.\n", treeNum, treeErrorMsg);
			exit(0);
		}
		tree->root->branch0->branch0=tree->root;
		tree->root->length0=tree->root->branch0->length0;
	}
	
	tree->totalLength=0.0;
	
	if (tree->rooted) {
		P=tree->root;
		while (P!=NULL) {
			tree->totalLength+=P->length0;
			P=P->branch1;
		}
	}

	if(storelike){
	  if((tree->likes=(double **)calloc(ncat+1,sizeof(double *)))==NULL){
	    fprintf(stderr, "Out of memory: site log likelihoods.");
	    exit(0);
	  }
	  for(i=0;i<ncat+1;i++){
	    if((tree->likes[i]=(double *)calloc(ngenes,sizeof(double)))==NULL){
	      fprintf(stderr, "Out of memory: site log likelihoods.");
	      exit(0);
	    }
	  }
	}
	else tree->likes=NULL;
 }


int IsTreeAvail(FILE *fv)
{
	char ch;

	ch=fgetc(fv);
	while (!feof(fv) && ch!='(' && ch!='[') 
		ch=fgetc(fv);
		
	if (ch=='(' || ch=='[')
		ungetc(ch, fv);
		
	return (!feof(fv));
}


void WriteNode(FILE *fv, TTree *tree, TNode *node)
{
	if (node->tipNo==-1) {
		fputc('(', fv);				
		WriteNode(fv, tree, node->branch1);
		fputc(',', fv);			
		WriteNode(fv, tree, node->branch2);
		fputc(')', fv);				
	} else {
	  fprintf(fv, "%s", tree->names[node->tipNo]);
	}
	if (tree->lengths)
	  fprintf(fv, ":%.12f", node->length0);

}


void WriteTree(FILE *fv, TTree *tree)
{
  
  copyedges(tree->root);/*I've been optimizing length1 and length2, not length0*/
  fputc('(', fv);	
  if (tree->rooted) {
    WriteNode(fv, tree, tree->root->branch1);
    fputc(',', fv);				
    WriteNode(fv, tree, tree->root->branch2);
  }
  else {
    WriteNode(fv, tree, tree->root->branch1);
    fputc(',', fv);				
    WriteNode(fv, tree, tree->root->branch2);
    fputc(',', fv);	
    WriteNode(fv, tree, tree->root->branch0);
  }
  fprintf(fv, ");\n");
}/*WriteTree*/

/*recursive: print conditional likelihoods at leaves*/
void print_CLike_node(TTree *tree, TNode *node, int ngenes, int nstates)
{
  int j,k;
  if(node->tipNo==-1){
    print_CLike_node(tree,node->branch1,ngenes,nstates);
    print_CLike_node(tree,node->branch2,ngenes,nstates);
  }
  else {
    printf("%s\n",tree->names[node->tipNo]);
    for(j=0;j<nstates;j++){
      for(k=0;k<ngenes;k++) printf("%f",node->CLike[0][j][k]);/*always use category 0, because same for all categories at leaves*/
      printf("\n");
    }
    printf("\n");
  }
}/*print_CLike_node*/


/*traverse tree and print conditional likelihoods at leaves*/
void print_CLike(TTree *tree, int ngenes, int nstates)
{
  printf("Conditional likelihoods at leaves\n");
  if (tree->rooted) {
		print_CLike_node(tree, tree->root->branch1,ngenes,nstates);
		print_CLike_node(tree, tree->root->branch2,ngenes,nstates);
	} else {
		print_CLike_node(tree, tree->root->branch1,ngenes,nstates);
		print_CLike_node(tree, tree->root->branch2,ngenes,nstates);
		print_CLike_node(tree, tree->root->branch0,ngenes,nstates);
	}
}/*print_CLike*/



void UnrootRTree(TTree *tree) 
/* Used to unroot a rooted tree */
{
	TNode *P, *Q, *R=NULL, *newRoot;
	double len, len2;
		
	if (!tree->rooted || tree->numTips<3)
		return;
	
	P=tree->tips[0];
	Q=P->branch0; newRoot=Q;
	while (Q!=tree->root) {
		R=Q->branch0;
		len=Q->length0;
		if (Q->branch1==P) {
			len2=Q->length1;
			Q->branch1=R;
			Q->length1=len;
		} else {
			len2=Q->length2;
			Q->branch2=R;
			Q->length2=len;
		}
		Q->branch0=P;
		Q->length0=len2;

		P=Q;
		Q=R;
	}

	len=R->length1+R->length2;
	if (R->branch1==P)
		Q=R->branch2;
	else
		Q=R->branch1;

	Q->branch0=P;
	Q->length0=len;
	
	if (P->branch1==R) {
		P->branch1=Q;
		P->length1=len;
	} else {
		P->branch2=Q;
		P->length2=len;
	}
	
	tree->root=newRoot;
	
	DisposeNode(R);
	
	tree->rooted=0;
}


void RerootUTree(TTree *tree, int tip) 
/* Used to reroot an unrooted tree */
/* This may sound strange but all it does is move
   the root trichotomy  */
{
	TNode *P, *Q, *R, *newRoot;
	double len, len2;
		
	if (tree->rooted)
		return;
	
	P=tree->tips[tip];
	Q=P->branch0; newRoot=Q;
	while (P!=tree->root) {
		R=Q->branch0;
		len=Q->length0;
		if (Q->branch1==P) {
			len2=Q->length1;
			Q->branch1=R;
			Q->length1=len;
		} else {
			len2=Q->length2;
			Q->branch2=R;
			Q->length2=len;
		}
		Q->branch0=P;
		Q->length0=len2;

		P=Q;
		Q=R;
	}
	 	
	tree->root=newRoot;
}

/*fill in the rate matrix indices on a rooted base tree
basenode is a node in the base tree,
i is the index of the category, and ratenode is a node on the rate tree, which contains the indices for rate matrices in this category on each edge (as edge lengths). Bad things will probably happen if basetree and ratetree don't have the same topology. maxQ is set to largest rate matrix index used anywhere*/
void fill_Qind(TNode *basenode, TNode *ratenode, int i, int *maxQ)
{
  if(basenode->branch1!=NULL) fill_Qind(basenode->branch1,ratenode->branch1,i,maxQ);/*left subtree*/
  if(basenode->branch2!=NULL) fill_Qind(basenode->branch2,ratenode->branch2,i,maxQ);/*right subtree*/
  if(basenode->tipNo==-1){/*not a leaf*/
    basenode->Qind1[i]=(int)ratenode->length1;/*cast to int because edge lengths are real in general, but must be integer when we're using them as rate matrix indices*/
    basenode->Qind2[i]=(int)ratenode->length2;
    if(*maxQ<basenode->Qind1[i]) *maxQ=basenode->Qind1[i];
    if(*maxQ<basenode->Qind2[i]) *maxQ=basenode->Qind2[i];
  }
  else{/*check names at leaf*/
    if(basenode->tipNo!=ratenode->tipNo){
      printf("Check: are these trees identical?\n");
    }
  }
}/*fill_Qind*/

/*WriteNode assumes the edge lengths are in length0, but I've been optimizing length1 and length2. Copy the optimized values to the right places. Call this with tree->root*/
void copyedges(TNode *node)
{
  if(node->branch1!=NULL) node->branch1->length0 = node->length1;
  if(node->branch2!=NULL) node->branch2->length0 = node->length2;

  if(node->branch1!=NULL) copyedges(node->branch1);
  if(node->branch2!=NULL) copyedges(node->branch2);
}/*copyedges*/
